/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonTexture.h"
#include "math.h"

NameValuePair modeConstants[] =
{
	{ "stretch", MULTIBLT_STRETCH },
	{ "tile", MULTIBLT_TILE },
	{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
	{ "tile-vertical", MULTIBLT_TILEVERTICAL },
	{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
	{ "tilevertical", MULTIBLT_TILEVERTICAL },
	{ 0, 0 }
};

NameValuePair borderValues[] = 
{
	{ ".none", 0 },
	{ "none", 0 },
	{ "raised", 1 },
	{ "sunken", 2 },
	{ "etched", 3 },
	{ "bump", 4 },
	{ 0, 0 }
};

NameValuePair gradientmodeConstants[] =
{
	{ ".none", GRADIENT_NONE },
	{ ".horizontal", GRADIENT_HORIZONTAL },
	{ ".vertical", GRADIENT_VERTICAL },
	{ ".radial", GRADIENT_RADIAL },
	{ ".diagonal", GRADIENT_DIAGONAL },
	{ ".fdiagonal", GRADIENT_FDIAGONAL },
	{ ".bdiagonal", GRADIENT_BDIAGONAL },
	{ 0, 0 }
};

NameValuePair gradienttransformationConstants[] =
{
	{ ".none", TRANSFORMATION_NONE },
	{ ".caricature", TRANSFORMATION_CHARICATURE },
	{ ".fisheye", TRANSFORMATION_FISHEYE },
	{ ".swirled", TRANSFORMATION_SWIRLED },
	{ ".cylinder", TRANSFORMATION_CYLINDER },
	{ ".shift", TRANSFORMATION_SHIFT },
	{ 0, 0 }
};

#define COLOR_GRADIENTACTIVECAPTION     27
#define COLOR_GRADIENTINACTIVECAPTION   28

ButtonTexture::ButtonTexture() :
	m_Data(NULL),

	transparencyNeeded(false),

	useAlphaMap(false),

	memoryNormalDC(0),
	memoryNormalBitmap(0),

	memoryHoverDC(0),
	memoryHoverBitmap(0),

	memoryHiddenDC(0),
	memoryHiddenBitmap(0),

	usenormalSolidColors(false),
	usehoverSolidColors(false),
	usehiddenSolidColors(false)
{
}

ButtonTexture::~ButtonTexture()
{
	if ( m_Data != NULL )
		delete m_Data;

	NormalGradient.clear();
	HoverGradient.clear();
	HiddenGradient.clear();

	if(memoryNormalBitmap)
	{
		memoryNormalBitmap = (HBITMAP) SelectObject(memoryNormalDC, memoryNormalBitmap);
		DeleteObject(memoryNormalBitmap);
	}
	if(memoryNormalDC)
		DeleteDC(memoryNormalDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryHiddenBitmap)
	{
		memoryHiddenBitmap = (HBITMAP) SelectObject(memoryHiddenDC, memoryHiddenBitmap);
		DeleteObject(memoryHiddenBitmap);
	}
	if(memoryHiddenDC)
		DeleteDC(memoryHiddenDC);
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType)
{
	bgcolor = RGB(255,255,255);
	lightcolor = GetSysColor(COLOR_3DLIGHT);
	darkcolor = GetSysColor(COLOR_3DSHADOW);
		
	if ( cfg.empty() )
		return;

	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);

	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		if (imageType == 0)
		{
			lightcolor = GetSysColor(COLOR_3DLIGHT);
			darkcolor = GetSysColor(COLOR_3DSHADOW);
		}
		else
		{
			lightcolor = normal.lightBevelColor;
			darkcolor = normal.darkBevelColor;
		}
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = lightcolor;
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color;
	COLORREF insertColor;
	char token[4096] = {0};
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			HiddenGradient.insert(HiddenGradient.end(), insertColor );
		}
	}
}

void ButtonTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel)
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = {left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void ButtonTexture::subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings)
{	
	workSettings.image = GetxRCString(type, "ButtonImage", defaultSettings.image);

	workSettings.leftEdge = GetxRCInt(type, "ButtonImageLeftEdge", defaultSettings.leftEdge, 0);
	workSettings.topEdge = GetxRCInt(type, "ButtonImageTopEdge", defaultSettings.topEdge, 0);
	workSettings.rightEdge = GetxRCInt(type, "ButtonImageRightEdge", defaultSettings.rightEdge, 0);
	workSettings.bottomEdge = GetxRCInt(type, "ButtonImageBottomEdge", defaultSettings.bottomEdge, 0);

	workSettings.mode = GetxRCNamedValue(type, "ButtonImageMode", modeConstants, defaultSettings.mode);

	workSettings.button_hueColor = GetxRCColor(type, "ButtonHueColor", defaultSettings.button_hueColor);
	workSettings.button_hueIntensity = GetxRCInt(type , "ButtonHueIntensity", defaultSettings.button_hueIntensity, 0, 100);
	workSettings.button_luminanceIntensity = GetxRCInt(type , "ButtonLuminanceIntensity", defaultSettings.button_luminanceIntensity, -100, 100);
	workSettings.button_saturationIntensity = GetxRCInt(type , "ButtonSaturationIntensity", defaultSettings.button_saturationIntensity, 0, 200);

	workSettings.button_mixColor = GetxRCColor(type, "ButtonMixColor", defaultSettings.button_mixColor);
	workSettings.button_mixIntensity = GetxRCInt(type , "ButtonMixIntensity", defaultSettings.button_mixIntensity, 0, 100);

	workSettings.solidColors = GetxRCLine(type, "ButtonSolidColors", defaultSettings.solidColors);
	workSettings.gradientColors = GetxRCLine(type, "ButtonSolidGradientColors", defaultSettings.gradientColors);

	workSettings.bevelsize = GetxRCLine(type, "ButtonSolidBevelSize", defaultSettings.bevelsize);

	workSettings.verticalGradient = GetxRCBool(type, "ButtonSolidGradientVertical", defaultSettings.verticalGradient);
	workSettings.repeatedGradient = GetxRCInt(type, "ButtonSolidGradientRepeated", defaultSettings.repeatedGradient, 0);

	workSettings.gradientType = GetxRCNamedValue(type, "ButtonSolidGradientType", gradientmodeConstants, defaultSettings.gradientType);
	workSettings.transformationType = GetxRCNamedValue(type, "ButtonSolidTransformationType", gradienttransformationConstants, defaultSettings.transformationType);

	workSettings.alphaTransparency = GetxRCInt(type, "ButtonAlphaTransparency", defaultSettings.alphaTransparency, 0, 255);
	workSettings.borderMethod = GetxRCNamedValue(type, "ButtonBorderMethod", borderValues, defaultSettings.borderMethod);
	workSettings.trueTransparent = GetxRCBool(type, "ButtonTrueTransparent", defaultSettings.trueTransparent);
}

void ButtonTexture::configure(const string &name, bool alphaMap)
{
	normal.image = GetxRCString(name, "ButtonImage", "");

	normal.leftEdge = GetxRCInt(name, "ButtonImageLeftEdge", 0, 0);
	normal.topEdge = GetxRCInt(name, "ButtonImageTopEdge", 0, 0);
	normal.rightEdge = GetxRCInt(name, "ButtonImageRightEdge", 0, 0);
	normal.bottomEdge = GetxRCInt(name, "ButtonImageBottomEdge", 0, 0);

	normal.mode = GetxRCNamedValue(name, "ButtonImageMode", modeConstants, MULTIBLT_STRETCH);

	normal.button_hueColor = GetxRCColor(name, "ButtonHueColor", RGB(255, 255, 255));
	normal.button_hueIntensity = GetxRCInt(name, "ButtonHueIntensity", 0, 0, 100);
	normal.button_luminanceIntensity = GetxRCInt(name, "ButtonLuminanceIntensity", 0, -100, 100);
	normal.button_saturationIntensity = GetxRCInt(name, "ButtonSaturationIntensity", 100, 0, 200);

	normal.button_mixColor = GetxRCColor(name, "ButtonMixColor", RGB(255, 255, 255));
	normal.button_mixIntensity = GetxRCInt(name, "ButtonMixIntensity", 0, 0, 100);

	normal.solidColors = GetxRCLine(name, "ButtonSolidColors", "");
	normal.gradientColors = GetxRCLine(name, "ButtonSolidGradientColors", "");

	normal.bevelsize = GetxRCLine(name, "ButtonSolidBevelSize", "0");

	normal.verticalGradient = GetxRCBool(name, "ButtonSolidGradientVertical");
	normal.repeatedGradient = GetxRCInt(name, "ButtonSolidGradientRepeated", 0);

	normal.gradientType = GetxRCNamedValue(name, "ButtonSolidGradientType", gradientmodeConstants, -1);
	normal.transformationType = GetxRCNamedValue(name, "ButtonSolidTransformationType", gradienttransformationConstants, -1);

	normal.alphaTransparency = GetxRCInt(name, "ButtonAlphaTransparency", 255, 0, 255);
	normal.borderMethod = GetxRCNamedValue(name, "ButtonBorderMethod", borderValues, 0);
	normal.trueTransparent = GetxRCBool(name, "ButtonTrueTransparent");

	useAlphaMap = alphaMap;

	subconfigure((name + "Hover"), hover, normal);

	subconfigure((name + "Hidden"), hidden, normal);

	usenormalSolidColors = GetxRCBool(name, "UseButtonSolidColors");

	if (usenormalSolidColors)
	{
		parseCFG(normal.solidColors, normal.backgroundColor, normal.lightBevelColor, normal.darkBevelColor, 0);
	
		//BG Insert in Gradient List
		NormalGradient.insert(NormalGradient.end(), normal.backgroundColor );

		if (normal.backgroundColor == RGB(255,0,255)|| normal.lightBevelColor == RGB(255,0,255) || normal.darkBevelColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		parseBevelCFG(normal.bevelsize, normal.bevelSizeLeft, normal.bevelSizeRight, normal.bevelSizeTop, normal.bevelSizeBottom);

		parseGradientCFG(normal.gradientColors, 0);
	}
	else
	{
		memoryNormalBitmap = LoadLSImage(normal.image.c_str(), 0);

		if (memoryNormalBitmap)
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryNormalBitmap, sizeof(BITMAP), &bm);
				
				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					PrepareAlphaChannel(memoryNormalBitmap, normal.button_hueColor, normal.button_hueIntensity, normal.button_luminanceIntensity, normal.button_saturationIntensity, normal.button_mixColor, normal.button_mixIntensity);
				}		
			}

			GetLSBitmapSize(memoryNormalBitmap, &imageNormalWidth, &imageNormalHeight);

			memoryNormalDC = CreateCompatibleDC(NULL);
			SelectObject(memoryNormalDC, memoryNormalBitmap);
		}
		else
		{
			transparencyNeeded = true;
		}
	}

	usehoverSolidColors = GetxRCBool(name, "UseHoverButtonSolidColors", usenormalSolidColors);

	if (usehoverSolidColors)
	{
		parseCFG(hover.solidColors, hover.backgroundColor, hover.lightBevelColor, hover.darkBevelColor, 1);
	
		//BG Insert in Gradient List
		HoverGradient.insert(HoverGradient.end(), hover.backgroundColor );

		if (hover.backgroundColor == RGB(255,0,255) || hover.lightBevelColor == RGB(255,0,255) || hover.darkBevelColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		parseBevelCFG(hover.bevelsize, hover.bevelSizeLeft, hover.bevelSizeRight, hover.bevelSizeTop, hover.bevelSizeBottom);

		parseGradientCFG(hover.gradientColors, 1);
	}
	else
	{
		if (!hover.image.empty())
		{
			memoryHoverBitmap = LoadLSImage(hover.image.c_str(), 0);

			if (memoryHoverBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryHoverBitmap, hover.button_hueColor, hover.button_hueIntensity, hover.button_luminanceIntensity, hover.button_saturationIntensity, hover.button_mixColor, hover.button_mixIntensity);
					}	
				}

				GetLSBitmapSize(memoryHoverBitmap, &imageHoverWidth, &imageHoverHeight);

				memoryHoverDC = CreateCompatibleDC(NULL);
				SelectObject(memoryHoverDC, memoryHoverBitmap);
			}
		}
	}

	usehiddenSolidColors = GetxRCBool(name, "UseHiddenButtonSolidColors", usenormalSolidColors);

	if (usehiddenSolidColors)
	{
		parseCFG(hidden.solidColors, hidden.backgroundColor, hidden.lightBevelColor, hidden.darkBevelColor, 2);
	
		//BG Insert in Gradient List
		HiddenGradient.insert(HiddenGradient.end(), hidden.backgroundColor );

		if (hidden.backgroundColor == RGB(255,0,255) || hidden.lightBevelColor == RGB(255,0,255) || hidden.darkBevelColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		parseBevelCFG(hidden.bevelsize, hidden.bevelSizeLeft, hidden.bevelSizeRight, hidden.bevelSizeTop, hidden.bevelSizeBottom);

		parseGradientCFG(hidden.gradientColors, 2);
	}
	else
	{
		if (!hidden.image.empty())
		{
			memoryHiddenBitmap = LoadLSImage(hidden.image.c_str(), 0);

			if (memoryHiddenBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryHiddenBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryHiddenBitmap, hidden.button_hueColor, hidden.button_hueIntensity, hidden.button_luminanceIntensity, hidden.button_saturationIntensity, hidden.button_mixColor, hidden.button_mixIntensity);
					}		
				}

				GetLSBitmapSize(memoryHiddenBitmap, &imageHiddenWidth, &imageHiddenHeight);

				memoryHiddenDC = CreateCompatibleDC(NULL);
				SelectObject(memoryHiddenDC, memoryHiddenBitmap);
			}
		}
	}

	if ( !useAlphaMap )
	{
		if (memoryNormalDC && !usenormalSolidColors)
			paintEffects(memoryNormalDC, imageNormalWidth, imageNormalHeight, normal.button_hueColor, normal.button_hueIntensity, normal.button_luminanceIntensity, normal.button_saturationIntensity, normal.button_mixColor, normal.button_mixIntensity);
		if (memoryHoverDC && !usehoverSolidColors)
			paintEffects(memoryHoverDC, imageHoverWidth, imageHoverHeight, hover.button_hueColor, hover.button_hueIntensity, hover.button_luminanceIntensity, hover.button_saturationIntensity, hover.button_mixColor, hover.button_mixIntensity);
		if (memoryHiddenDC && !usehiddenSolidColors)
			paintEffects(memoryHiddenDC, imageHiddenWidth, imageHiddenHeight, hidden.button_hueColor, hidden.button_hueIntensity, hidden.button_luminanceIntensity, hidden.button_saturationIntensity, hidden.button_mixColor, hidden.button_mixIntensity);
	}
}

bool ButtonTexture::useAlphaFeatures(int buttonType)
{
	if ( buttonType == 0 && normal.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 1 && hover.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 2 && hidden.alphaTransparency != 255 )
		return true;

	return false;
}

void ButtonTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	if ( width <= 0 || height <= 0 )
		return;

	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedAlphaTransparency = 255;
	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	bool solidColorsApplied = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (buttonType == 0 && usenormalSolidColors)
	{
		solidColorsApplied = true;

		usedAlphaTransparency = normal.alphaTransparency;
		usedBorderMethod = normal.borderMethod;
		useTrueTransparent = normal.trueTransparent;

		bevelWidth = normal.bevelSizeLeft+normal.bevelSizeRight;
		bevelHeight = normal.bevelSizeTop+normal.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, normal.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normal.bevelSizeRight;
			r.bottom = r.bottom-normal.bevelSizeBottom;
			::SetBkColor(bufferDC, normal.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normal.bevelSizeLeft;
			r.top = r.top+normal.bevelSizeTop;

			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 1 && usehoverSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = hover.alphaTransparency;
		usedBorderMethod = hover.borderMethod;
		useTrueTransparent = hover.trueTransparent;

		bevelWidth = hover.bevelSizeLeft+hover.bevelSizeRight;
		bevelHeight = hover.bevelSizeTop+hover.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hover.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hover.bevelSizeRight;
			r.bottom = r.bottom-hover.bevelSizeBottom;
			::SetBkColor(bufferDC, hover.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hover.bevelSizeLeft;
			r.top = r.top+hover.bevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 1);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 1);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && usehiddenSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = hidden.alphaTransparency;
		usedBorderMethod = hidden.borderMethod;
		useTrueTransparent = hidden.trueTransparent;

		bevelWidth = hidden.bevelSizeLeft+hidden.bevelSizeRight;
		bevelHeight = hidden.bevelSizeTop+hidden.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hidden.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hidden.bevelSizeRight;
			r.bottom = r.bottom-hidden.bevelSizeBottom;
			::SetBkColor(bufferDC, hidden.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hidden.bevelSizeLeft;
			r.top = r.top+hidden.bevelSizeTop;

			if ( HiddenGradient.size() > 1 )
			{
				if ( hidden.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HiddenGradient, r, 2);
				else
					DrawGradient(bufferDC, r, HiddenGradient[0], HiddenGradient[1], hidden.gradientType, hidden.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hidden.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HiddenGradient.size() > 1 )
			{
				if ( hidden.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HiddenGradient, r, 2);
				else
					DrawGradient(bufferDC, r, HiddenGradient[0], HiddenGradient[1], hidden.gradientType, hidden.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hidden.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if ( buttonType == 1 && memoryHoverBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 2 && memoryHiddenBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0 && memoryNormalBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0)
		{
			usedAlphaTransparency = normal.alphaTransparency;
			usedBorderMethod = normal.borderMethod;
			useTrueTransparent = normal.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normal.leftEdge, normal.topEdge, normal.rightEdge, normal.bottomEdge,
					normal.mode, SRCCOPY);
		}
		else if (buttonType == 1)
		{
			usedAlphaTransparency = hover.alphaTransparency;
			usedBorderMethod = hover.borderMethod;
			useTrueTransparent = hover.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hover.leftEdge, hover.topEdge, hover.rightEdge, hover.bottomEdge,
					hover.mode, SRCCOPY);
		}
		else if (buttonType == 2)
		{
			usedAlphaTransparency = hidden.alphaTransparency;
			usedBorderMethod = hidden.borderMethod;
			useTrueTransparent = hidden.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHiddenDC, 0, 0, imageHiddenWidth, imageHiddenHeight,
					hidden.leftEdge, hidden.topEdge, hidden.rightEdge, hidden.bottomEdge,
					hidden.mode, SRCCOPY);
		}
	}

	RECT completeFillRect = {0, 0, width, height};

	if (usedBorderMethod == 1)
		DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	
	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap || pPixel[3] == 0 || solidColorsApplied )
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, usedAlphaTransparency, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(r, g, b, hueh, hues, huel);
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				if ( needHSLConversion )
				{
					RGB2HSL(r, g, b, h, s, l);

					if(button_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
					if(button_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

					if(button_saturationIntensity != 100)
					{
						if ( button_saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
					}
				
					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix effect
					r = (BYTE)((r * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
				}
			}
			else
			{
				transparencyNeeded = true;
				if ( !needHSLConversion && !needMixColor )
					return;
			}
	
			if ( needHSLConversion || needMixColor )
				SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int imageType)
{
	bool vertical = false;
	int repeated = 0;

	if ( imageType == 0 )
	{
		vertical = normal.verticalGradient;
		repeated = normal.repeatedGradient;
	}
	else if( imageType == 1 )
	{
		vertical = hover.verticalGradient;
		repeated = hover.repeatedGradient;
	}
	else if( imageType == 2 )
	{
		vertical = hidden.verticalGradient;
		repeated = hidden.repeatedGradient;
	}
	
	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

void ButtonTexture::DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType)
{
	int i, j, y, x1, y1;
	double r, d, n1;

	// Calculate image size
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	int size = height*width*4;
	double radius = sqrt( pow(height/2,2) + pow(width/2,2) );
	double k = double(rect.bottom-rect.top)/double(rect.right-rect.left);
	if ( gradientType == GRADIENT_BDIAGONAL )
		k = double(rect.bottom-rect.top)/double(rect.left-rect.right);
	if ( ( gradientType == GRADIENT_FDIAGONAL ) || ( gradientType == GRADIENT_BDIAGONAL ) )
		gradientType = GRADIENT_DIAGONAL;
	double n = 0.0;
	double k1 = -1/k;

	// Create data buffer
	m_Data = new unsigned char[size];
	
	// Create gradient
	for ( i=0; i<height; i++ )
	{
		for ( j=0; j<width; j++ )
		{
			switch (gradientType)
			{
				// Horizontal gradient
				case GRADIENT_HORIZONTAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)j/(double)width)*GetBValue(endColor) + (1-(double)j/(double)width)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)j/(double)width)*GetGValue(endColor) + (1-(double)j/(double)width)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)j/(double)width)*GetRValue(endColor) + (1-(double)j/(double)width)*GetRValue(startColor));
				}
				break;

				// Vertical gradient
				case GRADIENT_VERTICAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)i/(double)height)*GetBValue(endColor) + (1-(double)i/(double)height)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)i/(double)height)*GetGValue(endColor) + (1-(double)i/(double)height)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)i/(double)height)*GetRValue(endColor) + (1-(double)i/(double)height)*GetRValue(startColor));
				}
				break;

				// Radial gradient
				case GRADIENT_RADIAL:
				{
					r = sqrt( pow(double(i-height/2)/radius,2) + pow(double(j-width/2)/radius,2) );
					if ( r <= 1.0 )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)(r*GetBValue(endColor) + (1-r)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)(r*GetGValue(endColor) + (1-r)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)(r*GetRValue(endColor) + (1-r)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = GetBValue(endColor);
						m_Data[i*width*4 + j*4+1] = GetGValue(endColor);
						m_Data[i*width*4 + j*4+2] = GetRValue(endColor);
					}
				}
				break;

				// Diagonal gradient
				case GRADIENT_DIAGONAL:
				{
					y = int(k*(j-width/2) + n);
					n1 = int((i-height/2) - k1*(j-width/2));
					x1 = int((n1-n)/(k-k1));
					y1 = int(k1*x1 + n1);
					d = sqrt( pow((j-width/2)-x1,2) + pow((i-height/2)-y1,2) ) / (2*radius);
					if ( (i-height/2) > y )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(endColor) + (0.5+d)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(endColor) + (0.5+d)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(endColor) + (0.5+d)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(startColor) + (0.5+d)*GetBValue(endColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(startColor) + (0.5+d)*GetGValue(endColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(startColor) + (0.5+d)*GetRValue(endColor));
					}
				}
				break;
			}
		}
	}

	if ( transformationType != TRANSFORMATION_NONE )
		ApplyTransformation( width, height, transformationType );

	// Create temporary bitmap
	HBITMAP hBitmap = (HBITMAP)CreateBitmap( width, height, 1, 32, m_Data );

	// Create temporary device context
	HDC hMemDC = CreateCompatibleDC(hDC);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// Blit gradient bitmap
	BitBlt( hDC, rect.left, rect.top, width, height, hMemDC, 0, 0, SRCCOPY );

	// Free memory
	SelectObject( hMemDC, hOldBitmap );
	DeleteDC(hMemDC);
	DeleteObject(hBitmap);
	delete m_Data;
	m_Data = NULL;
}

void ButtonTexture::ApplyTransformation(int width, int height, int transformationType)
{
	int X, Y;
	double r, fi;
	double R = sqrt( pow(width/4,2) + pow(height/4,2) );
	double FI = atan( double(height)/double(width) );

	int size = height*width*4;
	unsigned char* newBuffer = new unsigned char[size];
	memcpy( newBuffer, m_Data, size );

	for ( int i=0; i<height; i++ )
	{
		for ( int j=0; j<width; j++ )
		{
			switch (transformationType)
			{
				case TRANSFORMATION_CHARICATURE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(sqrt(r*R*2)*cos(fi));
					Y = int(fabs(sqrt(r*R*2)*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_FISHEYE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int((pow(r,2)/R)*cos(fi));
					Y = int(fabs((pow(r,2)/R)*sin(fi))) % height;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SWIRLED:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi+r/8)));
					Y = int(fabs(r*sin(fi+r/8)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_CYLINDER:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = abs((j-width/2) + int(fi+r/10)%32 - 8);
					Y = i;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SHIFT:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi)));
					Y = int(fabs(r*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;
			}
		}
	}

	memcpy( m_Data, newBuffer, size );
	delete newBuffer;
}

void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int i = 0; i < iSize; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		if ( needHSLConversion )
		{
			//AlphaMap Colorizing
			RGB2HSL(r, g, b, h, s, l);

			if(button_hueIntensity != 0)
				h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
			if(button_luminanceIntensity != 0)
				l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

			if(button_saturationIntensity != 100)
			{
				if ( button_saturationIntensity > 100 )
					s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
				else
					s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
			}

			HSL2RGB(r, g, b, h, s, l);
		}

		if ( needMixColor )
		{
			// HueMix effect
			r = (BYTE)((r * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}