/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
#define AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef vector<COLORREF> ColorRefList;
typedef vector<COLORREF>::iterator ColorRefListIterator;

#define	GRADIENT_NONE			-1
#define	GRADIENT_HORIZONTAL		0
#define	GRADIENT_VERTICAL		1
#define	GRADIENT_RADIAL			2
#define	GRADIENT_DIAGONAL		3
#define	GRADIENT_FDIAGONAL		4
#define	GRADIENT_BDIAGONAL		5

#define	TRANSFORMATION_NONE			-1
#define	TRANSFORMATION_CHARICATURE  0
#define	TRANSFORMATION_FISHEYE		1
#define	TRANSFORMATION_SWIRLED		2
#define	TRANSFORMATION_CYLINDER		3
#define	TRANSFORMATION_SHIFT		4

struct BUTTONSETTINGS
{
	string image;

	int leftEdge;
	int topEdge;
	int rightEdge;
	int bottomEdge;

	int mode;

	COLORREF button_hueColor;
	int button_hueIntensity;
	int button_luminanceIntensity;
	int button_saturationIntensity;

	COLORREF button_mixColor;
	int button_mixIntensity;

	string solidColors;
	string gradientColors;

	string bevelsize;

	bool verticalGradient;
	int repeatedGradient;

	int gradientType;
	int transformationType;

	//Later extracted
	//Out of bevelsize
	int bevelSizeLeft;
	int bevelSizeRight;
	int bevelSizeTop;
	int bevelSizeBottom;

	//Out of solidColors
	COLORREF backgroundColor;
	COLORREF darkBevelColor;
	COLORREF lightBevelColor;
	//-------------

	int alphaTransparency;
	int borderMethod;
	bool trueTransparent;

};

class ButtonTexture  
{
public:

	ButtonTexture();
	virtual ~ButtonTexture();

	void configure(const string &name, bool alphaMap);
	void applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType);

	bool isTransparencyNeeded(){ return transparencyNeeded; };

private:

	bool transparencyNeeded;

	bool useAlphaMap;
	bool useAlphaFeatures(int buttonType);

	void subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings);

	void parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType);
	void parseGradientCFG(string cfg, int imageType);
	void parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel);
	void solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int imageType);
	void paintEffects(HDC destHDC, int width, int height, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity );
	void PrepareAlphaChannel(HBITMAP &bmp, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity );

	unsigned char* m_Data;
	void DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType);
	void ApplyTransformation(int width, int height, int transformationType);

	ColorRefList NormalGradient;
	ColorRefList HoverGradient;
	ColorRefList HiddenGradient;

	HDC memoryNormalDC;
	HBITMAP memoryNormalBitmap;

	HDC memoryHoverDC;
	HBITMAP memoryHoverBitmap;

	HDC memoryHiddenDC;
	HBITMAP memoryHiddenBitmap;

	int imageNormalHeight;
	int imageNormalWidth;

	int imageHoverHeight;
	int imageHoverWidth;

	int imageHiddenHeight;
	int imageHiddenWidth;

	BUTTONSETTINGS normal;
	BUTTONSETTINGS hover;
	BUTTONSETTINGS hidden;

	bool usenormalSolidColors;
	
	bool usehoverSolidColors;

	bool usehiddenSolidColors;

};

#endif // !defined(AFX_BUTTONTEXTURE_H__4F10FDCD_EF77_423F_8223_011704DFFED2__INCLUDED_)
