/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "Tray.h"
#include "TrayTexture.h"
#include "ButtonTexture.h"
#include "TrayIcon.h"

#include "GhostTray.h"
#include "GhostButton.h"

#include "shlwapi.h"

extern int screenX;
extern int screenY;

extern int bitsperpixel;

//Alpha Transparency
typedef BOOL (WINAPI *PULW)(HWND hwnd, HDC hdcDst, POINT *pptDst, SIZE *psize, HDC hdcSrc, POINT *pptSrc, COLORREF crKey, BLENDFUNCTION *pblend, DWORD dwFlags);
static PULW pUpdateLayeredWindow = NULL;

#define TIMER_MOUSETRACK	1
#define TIMER_ALPHAFADEIN	2
#define TIMER_ALPHAFADEOUT	3
#define TIMER_CLEAN			4
#define TIMER_INFOTIPTIMEOUT 5
#define TIMER_SECURESTART	 6
#define TIMER_TRAYADD		 7
#define TIMER_TRAYREMOVE	 8

#define TIMER_ALPHAFADEUPBANG 12
#define TIMER_ALPHAFADEDOWNBANG 13

#define INFOTIMER_AUTOHIDE 100

UINT lsMessages[] = { 
						LM_SYSTRAY,
						LM_REFRESH,
						0
					 };

#define INFOTIP_MAXIMUM_WIDTH 300
#define INFOTIP_MAXIMUM_TIMEOUT 15000
#define INFOTIP_MINIMUM_TIMEOUT 7500

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

Tray::Tray(const string &name) :
	actualx(0),
	actualy(0),
	actualwidth(0),
	actualheight(0),
	
	hWnd(0),
	hInstance(0),
	box(0),

	ToolTips(0),

	oldbackupBMP(0),
	backupDC(0),
	backupBitmap(0),
	skipBackup(false),

	isFading(false),
	fadeBackupDC(NULL),
	fadeBackupBitmap(NULL),
	oldfadeBackupBMP(NULL),

	blocked(false),

	ParentDestroyNotificationReceived(false),

	disablezOrderChange(true),

	dontPaintAddedTrayIcon(NULL),

	AddedTrayIcon(NULL),
	RemovedTrayIcon(false),

	internalAddCount(0),

	InfoTiphWnd(0),
	infoTipsEnabled(false),

	startWithPosition(0),
	maxIconSize(16),

	autoSizeSteps(0),
	autoSizeDelay(0),
	
	backupExportedButtonCount(0),

	afterClass(""),
	afterTitle(""),

	lines_rows(1),
	lines_rowsAdded(0),

	wrapCount(-1),

	borderDrag(false),

	autoHidden(false),

	alphaInitialized(false),
	layeredFlag(false),

	alphaFade(false),
	UpdateAlpha(0),
	alphaFadeStep(2),
	alphaFadeDelay(10),

	mouseInside(false),

	focusOnEnter(false),
	skipButton(false),

	visible(false),

	visibleIcons(0),
	hiddenvisible(false),

	inCriticalProcess(false),

	capture(NULL),
	movecapture(NULL),
	changeState(NULL)
{
	this->name = name;
	
	UpdateConfig.x = 0;
	UpdateConfig.y = 0;
	UpdateConfig.button = NULL;

	traytexture = new TrayTexture();
	trayicon = new TrayIcon();
}

Tray::~Tray()
{
	HideList.clear();
	NoDragList.clear();
	SortList.clear();

	if (hWnd && ToolTips)
	{
		updateHint(hWnd, "", false, true);
		DestroyWindow(ToolTips);
		ToolTips = 0;
	}

	if ( showInfoTips )
		DestroyInfoTip();

	// release buttons
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
	{	
		delete (*tbit);
		*tbit = NULL;
	}
	traybuttons.clear();

	// unregister messages
	SendMessage( GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );

	if(IsWindow(hWnd))
	{	
		// shutdown timer
		KillTimer(hWnd, TIMER_MOUSETRACK);
		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		KillTimer(hWnd, TIMER_CLEAN);
		KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
		KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
		KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);

		DestroyWindow(hWnd);
	}

	if ( backupBitmap )
	{
		backupBitmap = (HBITMAP) SelectObject(backupDC, oldbackupBMP);
		DeleteObject(backupBitmap);
	}
	if ( backupDC )
		DeleteObject(backupDC);
	if ( oldbackupBMP )
		DeleteObject(oldbackupBMP);

	if(fadeBackupBitmap)
	{
		fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
		DeleteObject(fadeBackupBitmap);
	}
	if (fadeBackupDC)
		DeleteDC(fadeBackupDC);
	if ( oldfadeBackupBMP )
		DeleteObject(oldfadeBackupBMP);

	for(GhostTrayListIterator gtlit = ghosttrayList.begin(); gtlit != ghosttrayList.end(); gtlit++)
	{
		delete *gtlit;
		*gtlit = NULL;
	}

	ghosttrayList.clear();

	for(GhostButtonListIterator glit = ghostbuttonList.begin(); glit != ghostbuttonList.end(); glit++)
	{
		delete *glit;
		*glit = NULL;
	}

	ghostbuttonList.clear();

	if (traytexture)
		delete traytexture;
	if (trayicon)
		delete trayicon;
}

void Tray::load(HINSTANCE hInstance, HWND setbox)
{
	this->hInstance = hInstance;

	if (IsWindow(setbox))
	{
		box = setbox;
	}
	else
	{
		box = NULL;
	}
	
	if (IsWindow(hWnd))	// window already exists...
		return;

	hWnd = CreateWindowEx( box ? 0 : WS_EX_TOOLWINDOW,
							"xTray",
							name.c_str(),
							(box ? WS_CHILD : WS_POPUP),
							actualx, actualy,		
							actualwidth, actualheight,
							(box ? box : GetLitestepWnd()), 
							0,
							hInstance,
							this);
		
	if (hWnd)
	{
		PrefixedLSSetVariable(name, "CurrentHWND", IntToString( (int) hWnd ));

		SendMessage( GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM) hWnd, (LPARAM) ::lsMessages );
		
		SetWindowLong(hWnd, GWL_USERDATA, MAGIC_DWORD);

		reconfigure();

		if ( !hideIfEmpty || !autoHidden )
		{
			if ( startHidden )
				hide();
			else
				show();

			SetTimer(hWnd, TIMER_SECURESTART, 10000, NULL);

			blocked = true;
			
			SendMessage(GetLitestepWnd(), LM_SYSTRAYREADY, NULL, NULL);
	
			blocked = false;
		}
		
		blocked = true;

		if ( autoSize )
			autoSizeFunction();
		
		CreateLayout();

		blocked = false;

		visibleIcons = 0;
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{
			if ( !(*tbit)->xTrayHideChecked )
			{
				if ( IsOnTrayList((*tbit)->hOwnerWnd, HideList, false) )
					(*tbit)->userHidden = true;

				(*tbit)->xTrayHideChecked = true;
			}

			if ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) )
				visibleIcons++;
		}

		if ( wrapCount != -1 )
		{
			lines_rowsAdded = 0;

			int count = visibleIcons/wrapCount;
			int rest = visibleIcons%wrapCount;
				
			int todo = count-lines_rows;
			
			if ( count != 0 && rest != 0 )
				todo++;

			while ( todo > 0 && lines_rows+lines_rowsAdded+1 <= maxlines_rows )
			{
				lines_rowsAdded++;
				PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));

				if ( !trayLineAdd.empty() ) 
					LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

				todo--;
			}
		
			PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));
		}

		if ( !trayAdd.empty() )
		{
			int todo = visibleIcons;
			while ( todo > 0 )
			{
				PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons-todo+1 ));
				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		CreateLayout();
	}
	else
	{
		delete this;
	}
}

void Tray::reconfigure(bool refresh)
{
	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	SetWindowRgn(hWnd, NULL, true);

	NameValuePair directionValues[] = {
			{ ".right", 1 },
			{ ".left", 2 },
			{ ".bottom", 3 },
			{ ".top", 4 },
			{ 0, 0 }
			};

	NameValuePair wrapdirectionValues[] = {
			{ ".bottom", 1 },
			{ ".top", 2 },
			{ ".right", 3 },
			{ ".left", 4 },
			{ 0, 0 }
			};

	NameValuePair layoutValues[] = {
			{ ".horizontal", 0 },
			{ ".vertical", 1 },
			{ 0, 0 }
			};
	
	HideList.clear();
	NoDragList.clear();
	SortList.clear();

	GetxRCList("*xTrayHide", HideList);
	GetxRCList("*xTrayNoDragNDrop", NoDragList);
	GetxRCList("*xTrayPosition", SortList, true);

	char classname[1024], windowname[1024];
	char *zorderbuffers[] = {classname, windowname};

	if ( LCTokenize( GetxRCLine(name, "BehindWindow", "").c_str(), zorderbuffers, 2, NULL) >= 1 )
	{
		afterClass = classname;
		afterTitle = windowname;
	}

	nodragndrop = GetxRCBool(name, "DisableDragnDrop");

	borderDrag = GetxRCBool(name, "Moveable");

	hideIfEmpty = GetxRCBool(name, "HideIfEmpty");

	cleanUpInterval = GetxRCInt(name, "CleanUpInterval", 10000, 1000, 3600000);

	//Events/Actions
	trayLButtonUp = GetxRCLine(name, "OnLeftClickUp", "");
	trayLButtonDown = GetxRCLine(name, "OnLeftClickDown", "");
	trayMButtonUp = GetxRCLine(name, "OnMiddleClickUp", "");
	trayMButtonDown = GetxRCLine(name, "OnMiddleClickDown", "");
	trayRButtonUp = GetxRCLine(name, "OnRightClickUp", "");
	trayRButtonDown = GetxRCLine(name, "OnRightClickDown", "");

	fireClicksAlways = GetxRCBool(name, "AlwaysFireClickEvents");

	trayMWheelDown = GetxRCLine(name, "OnWheelDown", "");
	trayMWheelUp = GetxRCLine(name, "OnWheelUp", "");
	trayEnter = GetxRCLine(name, "OnEnter", "");
	trayLeave = GetxRCLine(name, "OnLeave", "");
	trayMove = GetxRCLine(name, "OnMove", "");
	trayResize = GetxRCLine(name, "OnResize", "");
	trayAdd = GetxRCLine(name, "OnAdd", "");
	trayRemove = GetxRCLine(name, "OnRemove", "");

	trayShow = GetxRCLine(name, "OnShow", "");
	trayHide = GetxRCLine(name, "OnHide", "");

	trayLineAdd = GetxRCLine(name, "OnLineAdd", "");
	trayLineRemove = GetxRCLine(name, "OnLineRemove", "");

	trayOnIconHide = GetxRCLine(name, "OnIconHide", "");
	trayOnIconUnHide = GetxRCLine(name, "OnIconUnHide", "");

	startHidden = GetxRCBool(name, "StartHidden");

	loopingTrayIcons = GetxRCBool(name, "LoopIcons");
	
	focusOnEnter = GetxRCBool(name, "FocusOnEnter");

	string borders = GetxRCLine(name, "Borders", "");
	if ( !borders.empty() )
	{
		char left[32], right[32], top[32], bottom[32];
		char *buffers[] = {left, right, top, bottom };

		int numTokens = LCTokenize(borders.c_str(), buffers, 4, NULL);
			
		if(numTokens == 1)
		{
			leftBorder = atoi(left);
			rightBorder = leftBorder;
			topBorder = leftBorder;
			bottomBorder = leftBorder;
		}
		else if (numTokens == 2)
		{
			leftBorder = atoi(left);
			rightBorder = leftBorder;
			topBorder = atoi(right);
			bottomBorder = topBorder;
		}
		else if (numTokens == 4)
		{
			leftBorder = atoi(left);
			rightBorder = atoi(right);
			topBorder = atoi(top);
			bottomBorder = atoi(bottom);
		}

		if ( leftBorder < 0 )
			leftBorder = 0;
		if ( rightBorder < 0 )
			rightBorder = 0;
		if ( topBorder < 0 )
			topBorder = 0;
		if ( bottomBorder < 0 )
			bottomBorder = 0;
	}
	else
	{
		leftBorder = GetxRCInt(name, "LeftBorder", 0, 0);
		topBorder = GetxRCInt(name, "TopBorder", 0, 0);
		rightBorder = GetxRCInt(name, "RightBorder", 0, 0);
		bottomBorder = GetxRCInt(name, "BottomBorder", 0, 0);
	}

	lines_rows = GetxRCInt(name, "Lines", 1, 1);
	maxlines_rows = GetxRCInt(name, "MaxLines", 100, 1);
	if (lines_rows > maxlines_rows)
		lines_rows = maxlines_rows;

	layout = GetxRCNamedValue(name, "Layout", layoutValues, 0);	
	
	wrapCount = GetxRCInt(name, "WrapCount", -1);
	if ( wrapCount <= 1 )
		wrapCount = -1;
	if ( wrapCount == -1 && lines_rows > 1 )
		wrapCount = 5;

	string tmpautoSize = GetxRCLine(name, "AutoSize", "0 0");
	if ( _stricmp(tmpautoSize.c_str(), "0 0") != NULL )
	{
		char step[32], delay[32];
		char *buffers[] = {step, delay};

		int numTokens = LCTokenize(tmpautoSize.c_str(), buffers, 2, NULL);
	
		autoSize = true;

		if ( numTokens >= 2 )
		{
			autoSizeSteps = atoi(step);
			autoSizeDelay = atoi(delay);

			if ( autoSizeSteps <= 0 || autoSizeDelay <= 0 )
			{
				autoSizeSteps = 0;
				autoSizeDelay = 0;
			}
			if ( autoSizeDelay > 50 )
			{
				autoSizeDelay = 50;
			}
		}
	}
	else
	{
		autoSize = false;
	}

	direction = GetxRCNamedValue(name, "Direction", directionValues, 1);
	wrapDirection = GetxRCNamedValue(name, "WrapDirection", wrapdirectionValues, 1);

	if ( layout == 0 )
	{
		if ( direction == 3 )
			direction = 1;
		else if ( direction == 4 )
			direction = 2;
		if ( wrapDirection == 3 )
			wrapDirection = 1;
		else if ( wrapDirection == 4 )
			wrapDirection = 2;
	}
	else if ( layout == 1 )
	{
		if ( direction == 1 )
			direction = 3;
		else if ( direction == 2 )
			direction = 4;
		if ( wrapDirection == 1 )
			wrapDirection = 3;
		else if ( wrapDirection == 2 )
			wrapDirection = 4;
	}

	xSpacing = GetxRCInt(name, "xSpacing", 0);
	ySpacing = GetxRCInt(name, "ySpacing", 0);

	transparencyMode = GetxRCBool(name, "TransparencyMode");

	if (refresh)
	{
		if (traytexture)
			delete traytexture;
		if (trayicon)
			delete trayicon;

		traytexture = 0;
		trayicon = 0;

		for(GhostTrayListIterator gtlit = ghosttrayList.begin(); gtlit != ghosttrayList.end(); gtlit++)
		{
			delete *gtlit;
			*gtlit = NULL;
		}

		ghosttrayList.clear();

		for(GhostButtonListIterator glit = ghostbuttonList.begin(); glit != ghostbuttonList.end(); glit++)
		{
			delete *glit;
			*glit = NULL;
		}

		ghostbuttonList.clear();

		traytexture = new TrayTexture();
		trayicon = new TrayIcon();
	}

	if ( IsOS(OS_2KXP) && GetxRCBool(name, "AlphaMap") && !GetxRCBool(name, "UseSolidColors") && !GetxRCString(name, "Image", "").empty() )
		traytexture->configure(name, true);
	else
		traytexture->configure(name, false);
	trayicon->configure(name);

	//Create Extra Painting
	StringList extraPainting = GetxRCLineList( "*xTrayOverlayTray" );
	
	for(StringListIterator sit = extraPainting.begin(); sit != extraPainting.end(); sit++)
	{
		numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
		
		if ( numTokens >= 5 )
		{
			GhostTray* newtray = new GhostTray();
		
			newtray->orgx = one;
			newtray->orgy = two;

			newtray->orgwidth = three;
			newtray->orgheight = four;
	
			if ( numTokens >= 6 )
			{
				if ( IsOS( OS_2KXP ) )
					newtray->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
			
				if ( numTokens == 7 && IsOS( OS_2KXP ) )
				{
					int tmpalpha = atoi( seven.c_str() );
					if ( tmpalpha < 0 || tmpalpha > 255 )
						newtray->alphatransparency = 255;
					else
						newtray->alphatransparency = tmpalpha;
				}
			}
	
			newtray->traytexture = new TrayTexture();
			newtray->traytexture->configure(five, newtray->alphamap);

			if ( newtray->traytexture != 0  )
				ghosttrayList.insert(ghosttrayList.end(), newtray);
			else
				delete newtray;
		}
	}
	
	//Create Extra Painting
	extraPainting = GetxRCLineList( "*xTrayOverlayButton" );
	
	for(sit = extraPainting.begin(); sit != extraPainting.end(); sit++)
	{
		//Use Split Helper Strings
		numTokens = SplitString((*sit), one, two, three, four, five, six, seven);
		
		if ( numTokens >= 5 )
		{
			GhostButton* newbutton = new GhostButton();
		
			newbutton->orgx = one;
			newbutton->orgy = two;

			newbutton->orgwidth = three;
			newbutton->orgheight = four;
		
			if ( numTokens >= 6 )
			{
				if ( IsOS( OS_2KXP ) )
					newbutton->alphamap = (strchr(six.c_str(), 'a') != 0 || strchr(six.c_str(), 'A') != 0);
			}

			newbutton->buttontexture = new ButtonTexture();
			newbutton->buttontexture->configure(five, newbutton->alphamap);

			if ( newbutton->buttontexture != 0 )
				ghostbuttonList.insert(ghostbuttonList.end(), newbutton);
			else
				delete newbutton;
		}
	}

	maxIconSize = GetxRCInt(name, "ButtonSize", trayicon->getIconSize(), 8, 64);

	if ( transparencyMode && !traytexture->isTransparencyNeeded() )
		transparencyMode = false;
	else if ( box && traytexture->isTransparencyNeeded() )
		transparencyMode = true;

	InfoTipColor = GetxRCColor(name, "ToolTipColor", GetSysColor(COLOR_INFOBK));
	InfoTipTextColor = GetxRCColor(name, "ToolTipTextColor", GetSysColor(COLOR_INFOTEXT));

	showToolTip = GetxRCBool(name, "ShowTooltip", true);

	if (!refresh)
	{
		int maxX = screenX;
		int maxY = screenY;
		if ( box )
		{
			RECT rc;
			GetClientRect(box, &rc);
			maxX = rc.right;
			maxY = rc.bottom;
		}
		
		actualx = GetxRCCoord(name, "X", 0, maxX);
		actualy = GetxRCCoord(name, "Y", 0, maxY);
		
		if ( !autoSize )
		{
			actualwidth = GetxRCDimen(name, "Width", leftBorder+maxIconSize+rightBorder, maxX);
			actualheight = GetxRCDimen(name, "Height", topBorder+maxIconSize+bottomBorder, maxY);
		}
		else
		{
			if ( autoSizeSteps != 0 && autoSizeDelay != 0 )
			{
				if ( direction <= 2 )
				{
					actualwidth = 0;
					actualheight = topBorder+maxIconSize+bottomBorder;
				}
				else
				{
					actualwidth = leftBorder+maxIconSize+rightBorder;
					actualheight = 0;
				}
			}
			else
			{
				actualwidth = 0;
				actualheight = 0;
			}
		}
		
		reposition(3, actualx, actualy, actualwidth, actualheight);

		if ( IsOS( OS_2KXP ) )
			showInfoTips = GetxRCBool(name, "ShowInfoTip");
		else
			showInfoTips = false;

		if ( showToolTip )
		{
			// struct specifying control classes to register
			INITCOMMONCONTROLSEX iccex; 

			// INITIALIZE COMMON CONTROLS 
			iccex.dwICC = ICC_WIN95_CLASSES;
			iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);

			InitCommonControlsEx(&iccex);
		}

		if ( showToolTip && showInfoTips )
		{ 
			if ( !CreateInfoTipWindow() )
				showInfoTips = false;
		}
		
		if ( showToolTip && !ToolTips )
		{
			ToolTips = CreateWindowEx(WS_EX_TOOLWINDOW, TOOLTIPS_CLASS, TEXT(""), WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP | ( (IsOS(OS_2KXP) && GetxRCBool(name, "BalloonTooltip")) ? TTS_BALLOON : 0),
									CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
									NULL, NULL, hInstance, NULL);

			SetWindowPos( ToolTips, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE ); 

			SendMessage( ToolTips, TTM_SETMAXTIPWIDTH, 0, INFOTIP_MAXIMUM_WIDTH );

			SendMessage( ToolTips, (UINT) TTM_SETTIPBKCOLOR, (WPARAM) (COLORREF) InfoTipColor, 0 );   
			SendMessage( ToolTips, (UINT) TTM_SETTIPTEXTCOLOR, (WPARAM) (COLORREF) InfoTipTextColor, 0);  

			char cone[32] = {0};
			char ctwo[32] = {0};
			char *cbuffers[] = {cone, ctwo};
			
			int numTokens = LCTokenize( (GetxRCLine(name, "TooltipDurations", "-1 -1")).c_str(), cbuffers, 2, NULL);
		
			int initial = -1;
			int autopop = -1;

			if ( numTokens >= 1 )
			{
				initial = atoi(cone);
				if ( numTokens >= 2 )
					autopop = atoi(ctwo);
			}

			if ( initial < 0 )
				initial = -1;
			if ( autopop < 1000 )
				autopop = 1000;

			if ( initial != -1 )
				SendMessage( ToolTips, (UINT) TTM_SETDELAYTIME,	(WPARAM) (DWORD) TTDT_INITIAL, (LPARAM) MAKELONG(initial, 0) );
			if ( autopop != -1 )
				SendMessage( ToolTips, (UINT) TTM_SETDELAYTIME,	(WPARAM) (DWORD) TTDT_AUTOPOP, (LPARAM) MAKELONG(autopop, 0) );

			if ( ToolTips )
			{
				RECT clientRect;
				GetClientRect(hWnd, &clientRect);
				
				TOOLINFO ti;
				ti.cbSize = sizeof(TOOLINFO);
				ti.uFlags = TTF_TRANSPARENT;
				ti.hwnd = hWnd;
				ti.uId = 0;
				ti.rect = clientRect;
				ti.hinst = hInstance;
				ti.lpszText = 0;
				ti.lParam = 0;

				SendMessage(ToolTips, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);
	
				updateHint(hWnd, "", true);
			}
		}
	}

	alphaTransparency = GetxRCInt(name, "AlphaTransparency", 255, 0, 255);

	if ( !GetxRCBool(name, "UseSolidColors") && !GetxRCString(name, "Image", "").empty() )
		alphaMap = GetxRCBool(name, "AlphaMap");
	else
		alphaMap = false;

	char step[32], time[32];
	char *alphabuffers[] = {step, time};

	string fade = GetxRCLine(name, "CustomAlphaFade", "");
	
	numTokens = LCTokenize( fade.c_str(), alphabuffers, 2, NULL);

	if (fade.empty())
	{
		alphaFade = false;
	}
	else if (numTokens == 1)
	{
		alphaFadeStep = atoi( step );
		if (alphaFadeStep < 1 || alphaFadeStep > 255)
			alphaFadeStep = 2;
		alphaFade = true;
	}
	else if (numTokens == 2)
	{
		alphaFadeStep = atoi( step );
		if (alphaFadeStep < 1 || alphaFadeStep > 255)
			alphaFadeStep = 2;
		alphaFadeDelay = atoi( time );
		if (alphaFadeDelay < 1 || alphaFadeDelay > 500)
			alphaFadeDelay = 10;
		alphaFade = true;
	}

	if (!alphaFade)
		alphaFade = GetxRCBool(name, "AlphaFade");

	if ( IsOS(OS_2KXP) && !refresh && (alphaTransparency != 255 || alphaFade || alphaMap))
	{
		ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

		if ( !alphaFade )
			UpdateWindowTransparent(alphaTransparency);
	}
	else if ( !IsOS(OS_2KXP) && !refresh)
	{
		alphaTransparency = 255;
		alphaFade = false;
		alphaMap = false;
	}

	if (!refresh)
	{
		setAlwaysOnTop( GetxRCBool(name, "AlwaysOnTop") );
	}
	else
	{
		blocked = true;

		if ( autoSize )
			autoSizeFunction();
		
		CreateLayout();

		blocked = false;

		if ( !trayLineRemove.empty() ) 
		{
			while ( lines_rowsAdded > 0 )
			{
				lines_rowsAdded--;
				PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));

				LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
			}
		}

		if ( !trayRemove.empty() )
		{
			int todo = visibleIcons;
			while ( todo > 0 )
			{
				visibleIcons--;
				PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons ));
				
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		visibleIcons = 0;
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{
			if ( !(*tbit)->xTrayHideChecked )
			{
				if ( IsOnTrayList((*tbit)->hOwnerWnd, HideList, false) )
					(*tbit)->userHidden = true;

				(*tbit)->xTrayHideChecked = true;
			}

			if ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) )
				visibleIcons++;
		}

		if ( wrapCount != -1 )
		{
			lines_rowsAdded = 0;

			int count = visibleIcons/wrapCount;
			int rest = visibleIcons%wrapCount;
			
			int todo = count-lines_rows;
		
			if ( count != 0 && rest != 0 )
				todo++;

			while ( todo > 0 && lines_rows+lines_rowsAdded+1 <= maxlines_rows )
			{
				lines_rowsAdded++;
				PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));

				if ( !trayLineAdd.empty() ) 
					LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

				todo--;
			}
	
			PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));
		}
		
		if ( !trayAdd.empty() )
		{
			int todo = visibleIcons;
			while ( todo > 0 )
			{
				PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons-todo+1 ));

				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				todo--;
			}
		}

		CreateLayout();
	}
}
	
void Tray::setAlwaysOnTop(bool setAlwaysOnTop)
{
	if (box)
		return;
	
	alwaysOnTop = setAlwaysOnTop;
	
	ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
	SetParent(hWnd, setAlwaysOnTop ? 0 : GetLitestepDesktop());
	ModifyStyle(hWnd, WS_CHILD, WS_POPUP);
		
	disablezOrderChange = false;

	if ( !afterClass.empty() )
	{
		HWND before = FindWindowEx2(afterClass, afterTitle);
		
		if ( before != NULL )
		{
			if ( alwaysOnTop )
			{
				SetWindowPos(hWnd, HWND_TOPMOST,
								0, 0, 0, 0,
								SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );

				disablezOrderChange = false;
			}

			SetWindowPos(hWnd, before,
							0, 0, 0, 0,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
		}
		else
		{
			SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
							0, 0, 0, 0,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
		}
	}
	else
	{
		SetWindowPos(hWnd, alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
						0, 0, 0, 0,
						SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
	}
}

void Tray::setAlpha(int newAlphaTransparency)
{
	if (box)
		return;

	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			layeredFlag = true;

			repaint();
		}

		alphaTransparency = newAlphaTransparency;

		UpdateWindowTransparent(newAlphaTransparency);
	}
}

void Tray::setAlphaFade(int newAlphaTransparency, int step, int delay)
{
	if (box)
		return;

	if ( newAlphaTransparency >= 0 && newAlphaTransparency <= 255 && IsOS(OS_2KXP) )
	{
		if (!layeredFlag)
		{
			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, 0);
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			ModifyStyleEx(hWnd, 0, WS_EX_LAYERED);

			ModifyStyle(hWnd, WS_POPUP, WS_CHILD);
			SetParent(hWnd, alwaysOnTop ? 0 : GetLitestepDesktop());
			ModifyStyle(hWnd, WS_CHILD, WS_POPUP);

			layeredFlag = true;

			repaint();
		}
		
		UpdateAlpha = alphaTransparency;
		
		alphaFadeBangEndValue = newAlphaTransparency;

		if ( UpdateAlpha > alphaFadeBangEndValue )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
			SetTimer(hWnd, TIMER_ALPHAFADEDOWNBANG, delay, NULL);
		}
		else if ( UpdateAlpha < alphaFadeBangEndValue )
		{
			KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
			SetTimer(hWnd, TIMER_ALPHAFADEUPBANG, delay, NULL);
		}
	}
}

void Tray::UpdateWindowTransparent(int newAlphaTransparency)
{
	/* First, see if we can get the API call we need. If we've tried
    * once, we don't need to try again. */
	if (!alphaInitialized)
	{
		HMODULE hDLL = LoadLibrary ("user32");

		pUpdateLayeredWindow = 
					(PULW) GetProcAddress(hDLL, "UpdateLayeredWindow");

		alphaInitialized = true;
   
		if (pUpdateLayeredWindow == NULL)
		{
		    ModifyStyleEx(hWnd, WS_EX_LAYERED, 0 );

			alphaTransparency = 255;
			alphaMap = false;
			alphaFade = false;

			layeredFlag = false;

			return;
		}
		else
		{
			layeredFlag = true;

			SetWindowRgn(hWnd, NULL, true);
		}
	}

	/* Now, we need to set the 'layered window attributes'. This
	 * is where the alpha values get set. */
	//------------------------------------------------------------------
	RECT rectPosition;
	GetWindowRect(hWnd, &rectPosition);
		
	int width = (rectPosition.right-rectPosition.left);
	int height = (rectPosition.bottom-rectPosition.top);

	if ( width <= 0 || height <= 0 )
		return;

	int x = rectPosition.left;
	int y = rectPosition.top;

	HDC	backgroundDC = CreateCompatibleDC(NULL);
			
	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP	backgroundBitmap = CreateDIBSection( backgroundDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

	// Use Complete Backup for WindowsInternal Repaint (Covering Windows, ...)
	if ( alphaFade && isFading && fadeBackupBitmap != 0 && fadeBackupDC != 0 )
	{
		BitBlt(backgroundDC, 0, 0, width, height, fadeBackupDC, 0, 0, SRCCOPY);
	}
	else
	//1----------------------------------------------------------------
	{
	//1----------------------------------------------------------------

		if ( !alphaMap )
		{
			//True Transparency
			if (transparencyMode)
			{
				rectPosition.left = 0;
				rectPosition.top = 0;
				rectPosition.right = width;
				rectPosition.bottom = height;
				::SetBkColor(backgroundDC, RGB(255, 0, 255));
				::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &rectPosition, NULL, 0, NULL);
			}
			//PrePaint Background (Fake Transparency)
			else if (!transparencyMode)
			{
				//  =======================================================================
				//  Paint desktop on display DC and then into background buffer
				//  =======================================================================
				PaintDesktopEx(backgroundDC, 0, 0, width, height, x, y, false);
			}
		}

		//Normal Background Painting
		traytexture->apply(backgroundDC, 0, 0, width, height, true);

		if ( !ghosttrayList.empty() )
		{
			for(GhostTrayListIterator glit = ghosttrayList.begin(); glit != ghosttrayList.end(); glit++)
			{
				if ( *glit != NULL )
				{
					int gtx = xParseCoord((*glit)->orgx, 0, width);
					int gty = xParseCoord((*glit)->orgy, 0, height);
					int gtwidth = xParseDimen((*glit)->orgwidth, width, width);
					int gtheight = xParseDimen((*glit)->orgheight, height, height);

					(*glit)->traytexture->applyOverlay(backgroundDC, gtx, gty, gtwidth, gtheight, (*glit)->alphatransparency );
				}
				else
				{
					break;
				}
			}
		}

		bool securecapture = false;
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{
			if ( (*tbit) != dontPaintAddedTrayIcon )
			{
				if ( ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) ) && (*tbit) != capture )
					(*tbit)->onPaint( backgroundDC, leftBorder, topBorder, width-rightBorder, height-bottomBorder );
				else if ( (*tbit) == capture )
					securecapture = true;
			}
		}

		if ( securecapture && ( !capture->userHidden || (capture->userHidden && hiddenvisible) ) )
			capture->onPaint( backgroundDC, leftBorder, topBorder, width-rightBorder, height-bottomBorder );

		//Prepare Combined Image
		BITMAP bm;
		GetObject(backgroundBitmap, sizeof(BITMAP), &bm);

		BYTE *pPixel = (LPBYTE)bm.bmBits;

		int iSize = bm.bmHeight * bm.bmWidth;

		for (int j = 0; j < iSize; ++j)
		{
			//Convert Text (and Normal Images) to Full Opacity
			if (pPixel[3] == 0)
				pPixel[3] = 255;
				
			if ( alphaMap )
			{
				//Remove Prepared Alpha (1 back to 0)
				if (pPixel[3] == 1)
					pPixel[3] = 0;
			}
			else
			{
				if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
				{
					pPixel[0] = 0;
					pPixel[1] = 0;
					pPixel[2] = 0;
					pPixel[3] = 0;
				}
				else
				{
					pPixel[3] = 255;
				}
			}

			pPixel += 4;
		}
	
		//Create fadeBackupBitmap, if it doesn't exist!
		//----------------------------------------------------------------------
		if ( alphaFade && isFading && fadeBackupBitmap == 0 && fadeBackupDC == 0 )
		{
			fadeBackupDC = CreateCompatibleDC(NULL);
				
			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=width;      // Set size you need
			alpha.bmiHeader.biHeight=height;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			fadeBackupBitmap = CreateDIBSection( fadeBackupDC,
														&alpha,
														DIB_RGB_COLORS,
														&pvBits,
														NULL,
														0 );

			oldfadeBackupBMP = (HBITMAP) SelectObject(fadeBackupDC, fadeBackupBitmap);

			BitBlt(fadeBackupDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);
		}
		//----------------------------------------------------------------------

	//1
	}
	//1}

	POINT ptWindowScreenPosition = {x, y};
	SIZE szWindow = {width, height};

	POINT ptSrc = {0,0};

	BLENDFUNCTION blendPixelFunction = { AC_SRC_OVER, 0, newAlphaTransparency, AC_SRC_ALPHA };

	pUpdateLayeredWindow(hWnd, NULL, &ptWindowScreenPosition, &szWindow, backgroundDC, &ptSrc, 0, &blendPixelFunction, ULW_ALPHA );
	//------------------------------------------------------------------*/

	if ( !afterClass.empty() )
	{
		HWND before = FindWindowEx2(afterClass, afterTitle);
			
		if ( before != NULL )
		{
			disablezOrderChange = false;

			SetWindowPos(hWnd, before,
							0, 0, 0, 0,
							SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE );
		}
	}

	if ( backgroundBitmap != 0 && backgroundDC != 0 )
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldBMP);
		DeleteObject(backgroundBitmap);
		DeleteDC(backgroundDC);
	}
}

//------------------------------------------------------------
//Show/Hide
//------------------------------------------------------------
void Tray::show()
{
	if ( ParentDestroyNotificationReceived )
		return;

	if (alphaFade && !visible)
	{
		KillTimer(hWnd, TIMER_ALPHAFADEOUT);
		SetTimer(hWnd, TIMER_ALPHAFADEIN, alphaFadeDelay, NULL);
	}
	else if ( !visible )
	{
		ShowWindow(hWnd, SW_SHOWNOACTIVATE);
	}

	visible = true;

	repaint();

	if ( !trayShow.empty() )
		LSExecute(hWnd, trayShow.c_str(), SW_SHOWNORMAL);
}

void Tray::hide()
{
	if (alphaFade && visible)
	{
		UpdateAlpha = alphaTransparency;

		KillTimer(hWnd, TIMER_ALPHAFADEIN);
		SetTimer(hWnd, TIMER_ALPHAFADEOUT, alphaFadeDelay, NULL);
	}
	else if ( visible )
	{
		ShowWindow(hWnd, SW_HIDE);
	}

	visible = false;

	if ( !trayHide.empty() )
		LSExecute(hWnd, trayHide.c_str(), SW_SHOWNORMAL);
}

void Tray::ToggleHiddenIcons()
{
	int changingIcons = 0;
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
	{
		if ( (*tbit)->userHidden )
			changingIcons++;
	}

	if ( changingIcons == 0 )
		return;

	if ( hiddenvisible )
	{
		if ( wrapCount != -1 )
		{
			int count = changingIcons/wrapCount;
			int rest = changingIcons%wrapCount;
			int rest1 = visibleIcons%wrapCount;
	
			int todo = count;

			if ( changingIcons-(count*wrapCount) >= rest1 )
				todo++;

			while ( todo > 0 && lines_rowsAdded > 0 )
			{	
				lines_rowsAdded--;
				PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));
	
				if ( !trayLineRemove.empty() )
					LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
				
				if ( lines_rowsAdded <= 0 )
				{
					lines_rowsAdded = 0;
					break;
				}

				todo--;
			}
		}
		
		if ( !trayRemove.empty() )
		{
			while ( changingIcons > 0 )
			{
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);
				changingIcons--;
			}
		}

		hiddenvisible = false;
	}
	else
	{
		int usechangingIcons = changingIcons;

		if ( wrapCount != -1 )
		{
			int correct = visibleIcons-(wrapCount*lines_rows);
			if ( correct < 0 )
				usechangingIcons = changingIcons+correct;

			if ( usechangingIcons > 0 )
			{
				int count = usechangingIcons/wrapCount;
				int rest = usechangingIcons%wrapCount;
				int rest1 = visibleIcons%wrapCount;

				int todo = count;

				if ( usechangingIcons-(count*wrapCount) >= rest1 )
					todo++;

				while ( todo > 0 && lines_rows+lines_rowsAdded+1 <= maxlines_rows )
				{	
					lines_rowsAdded++;
					PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows+lines_rowsAdded ));
		
					if ( !trayLineAdd.empty() )
						LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

					todo--;
				}
			}
		}
		
		if ( !trayAdd.empty() )
		{
			while ( changingIcons > 0 )
			{
				LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
				changingIcons--;
			}
		}

		hiddenvisible = true;	
	}
	
	blocked = true;

	if ( autoSize )
		autoSizeFunction();
	
	CreateLayout();

	blocked = false;
}

//------------------------------------------------------------
//Move/Resize/Reposition
//------------------------------------------------------------
//Type == 1 -> Move
//Type == 2 -> Resize
//Type == 3 -> Reposition

void Tray::reposition(int type, int newx, int newy, int newwidth, int newheight, int steps, int time, bool repositionBy)
{
	int x = actualx;
	int y = actualy;
	
	int width = actualwidth;
	int height = actualheight;

	if (steps > 0 && time > 0)
	{
		if (time > 50)
			time = 50;

		if (repositionBy)
		{
			if (type != 2)
			{
				newx = x + newx;
				newy = y + newy;
			}
			if (type != 1)
			{
				newwidth = width + newwidth;
				newheight = height + newheight;
			}
		}

		skipBackup = true;

		int counter = 0;
		int deltax = newx - x;
		int deltay = newy - y;
		int deltasizex = newwidth - width;
		int deltasizey = newheight - height;

		bool growx;
		bool growy;
		bool growsizex;
		bool growsizey;
		
		if (deltax < 0)
			growx = false;
		else
			growx = true;
		if (deltay < 0)
			growy = false;
		else
			growy = true;
		
		if (deltasizex < 0)
			growsizex = false;
		else
			growsizex = true;
		if (deltasizey < 0)
			growsizey = false;
		else
			growsizey = true;

		int restx = 0;
		int resty = 0;
		int restw = 0;
		int resth = 0;

		div_t div_result;
		div_result = div( deltax, steps );
		deltax = div_result.quot;
		restx = div_result.rem;
		div_result = div( deltay, steps );
		deltay = div_result.quot;
		resty = div_result.rem;

		div_result = div( deltasizex, steps );
		deltasizex = div_result.quot;
		restw = div_result.rem;
		div_result = div( deltasizey, steps );
		deltasizey = div_result.quot;
		resth = div_result.rem;

		if (deltax == 0 && growx)
			deltax = 1;
		else if (deltax == 0 && !growx)
			deltax = -1;
		if (deltay == 0 && growy)
			deltay = 1;
		else if (deltay == 0 && !growy)
			deltay = -1;

		if (deltasizex == 0 && growsizex)
			deltasizex = 1;
		else if (deltasizex == 0 && !growsizex)
			deltasizex = -1;
		if (deltasizey == 0 && growsizey)
			deltasizey = 1;
		else if (deltasizey == 0 && !growsizey)
			deltasizey = -1;

		int dynx = x;
		int dyny = y;
		int dynwidth = width;
		int dynheight = height;

		int stopper = 0;
		while (counter <= steps)
		{
			stopper = 0; // Stop if x, y, width and height are finished, stopper = 4
			if ( restx > 0 && deltax > 0 )
			{
				dynx = dynx+deltax+1;
				restx--;
			}
			else if ( restx < 0 && deltax < 0 )
			{
				dynx = dynx+deltax-1;
				restx++;
			}
			else
				dynx = dynx+deltax;

			if (growx && dynx > newx)
			{
				dynx = newx;
				stopper++;
			}
			else if (!growx && dynx < newx)
			{
				dynx = newx;
				stopper++;
			}

			if ( resty > 0 && deltay > 0 )
			{
				dyny = dyny+deltay+1;
				resty--;
			}
			else if ( resty < 0 && deltay < 0 )
			{
				dyny = dyny+deltay-1;
				resty++;
			}
			else
				dyny = dyny+deltay;

			if (growy && dyny > newy)
			{
				dyny = newy;
				stopper++;
			}
			else if (!growy && dyny < newy)
			{
				dyny = newy;
				stopper++;
			}

			if ( restw > 0 && deltasizex > 0 )
			{
				dynwidth = dynwidth+deltasizex+1;
				restw--;
			}
			else if ( restw < 0 && deltasizex < 0 )
			{
				dynwidth = dynwidth+deltasizex-1;
				restw++;
			}
			else
				dynwidth = dynwidth+deltasizex;

			if (growsizex && dynwidth > newwidth)
			{
				dynwidth = newwidth;
				stopper++;
			}
			else if (!growsizex && dynwidth < newwidth)
			{
				dynwidth = newwidth;
				stopper++;
			}

			if ( resth > 0 && deltasizey > 0 )
			{
				dynheight = dynheight+deltasizey+1;
				resth--;
			}
			else if ( resth < 0 && deltasizey < 0 )
			{
				dynheight = dynheight+deltasizey-1;
				resth++;
			}
			else
				dynheight = dynheight+deltasizey;

			if (growsizey && dynheight > newheight)
			{
				dynheight = newheight;
				stopper++;
			}
			else if (!growsizey && dynheight < newheight)
			{
				dynheight = newheight;
				stopper++;
			}

			if ( type == 2 || type == 3 )
				CreateLayout(false, dynwidth, dynheight);
				
			SetWindowPos(hWnd, 0, dynx, dyny, dynwidth, dynheight,
					SWP_NOACTIVATE | (type == 2 ? SWP_NOMOVE : 0) | (type == 1 ? SWP_NOSIZE : 0 ) | SWP_NOZORDER);
				
			if (box != NULL)
				RedrawWindow(box, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
			
			repaint();

			if (stopper == 4)
				counter = steps;

			counter++;

			Sleep(time);
		}

		//Paint Added TrayIcon with the last Resize Action
		dontPaintAddedTrayIcon = NULL;

		skipBackup = false;

		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			CreateLayout(false, newwidth, newheight);
			
			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);

			repaint();
		}
		else if ( type == 3 )
		{
			CreateLayout(false, newwidth, newheight);
			
			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);

			repaint();
		}
	}
	else
	{
		if (repositionBy)
		{
			if (type != 2)
			{
				newx = x + newx;
				newy = y + newy;
			}
			if (type != 1)
			{
				newwidth = width + newwidth;
				newheight = height + newheight;
			}
		}

		//Paint Added TrayIcon with the last Resize Action
		dontPaintAddedTrayIcon = NULL;

		if ( type == 1 )
		{
			SetWindowPos(hWnd, 0, newx, newy, 0, 0,
					SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
		}
		else if ( type == 2 )
		{
			CreateLayout(false, newwidth, newheight);

			SetWindowPos(hWnd, 0, 0, 0, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER);
		}
		else if ( type == 3 )
		{
			CreateLayout(false, newwidth, newheight);

			SetWindowPos(hWnd, 0, newx, newy, newwidth, newheight,
					SWP_NOACTIVATE | SWP_NOZORDER);
		}
	}
}

void Tray::updateRegion()
{	
	RECT rectPosition;
	GetClientRect(hWnd, &rectPosition);

	HDC tmpDC = CreateCompatibleDC(NULL);
	HBITMAP tmpBmp = CreateBitmap(rectPosition.right, rectPosition.bottom, 1, bitsperpixel, NULL);
	HBITMAP oldBMP = (HBITMAP) SelectObject(tmpDC, tmpBmp);

	onPaint(tmpDC);
								
	tmpBmp = (HBITMAP) SelectObject(tmpDC, oldBMP);

	HRGN region = BitmapToRegion(tmpBmp, RGB(255, 0, 255), 0, 0, 0);
	HRGN windowRgn = CreateRectRgn(0, 0, 0, 0);
	CombineRgn(windowRgn, region, NULL, RGN_COPY);

	SetWindowRgn(hWnd, windowRgn, true);

	DeleteObject(windowRgn);
	DeleteObject(region);

	DeleteObject(tmpBmp);
	DeleteDC(tmpDC);
}

//------------------------------------------------------------
//Mouse Related Actions (Clicks, Enter, Leave, Move, ...)
//------------------------------------------------------------

TrayButton* Tray::onLButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onLButtonDown(int x, int y, LPARAM lParam)
{  
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button != NULL && !nodragndrop )
	{
		if ( button->noDragnDrop == -1 )
		{
			if ( IsOnTrayList(button->hOwnerWnd, NoDragList, false) )
			{
				movecapture = NULL;
				button->noDragnDrop = 0;
			}
			else
			{
				movecapture = button;
				button->noDragnDrop = 1;
			}
		}
		else if ( button->noDragnDrop == 1 )
		{
			movecapture = button;
		}
	}
	
	if ( (button == NULL || fireClicksAlways) && !trayLButtonDown.empty() )
	{
		LSExecute(hWnd, trayLButtonDown.c_str(), SW_SHOWNORMAL);
	}
	
	if ( button == NULL && borderDrag )
	{
		SendMessage(hWnd, WM_SYSCOMMAND, SC_MOVE | 2, lParam);
	}

	return button;
}

TrayButton* Tray::onLButtonUp(int x, int y)
{	
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button != NULL && !nodragndrop && movecapture != NULL && movecapture != button )
	{	
		if ( button->noDragnDrop == -1 )
		{
			if ( IsOnTrayList(button->hOwnerWnd, NoDragList, false) )
			{
				movecapture = NULL;
				button->noDragnDrop = 0;
			}
			else
			{
				button->noDragnDrop = 1;
			}
		}
		
		if ( button->noDragnDrop == 1 )
		{
			TrayButtonList tmptraybuttons;
			
			for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
			{
				if ( *tbit == button )
				{
					if ( button->positionInTray < movecapture->positionInTray )
					{
						tmptraybuttons.insert( tmptraybuttons.end(), (movecapture) );
						tmptraybuttons.insert( tmptraybuttons.end(), (button) );
					}
					else if ( button->positionInTray > movecapture->positionInTray )
					{
						tmptraybuttons.insert( tmptraybuttons.end(), (button) );
						tmptraybuttons.insert( tmptraybuttons.end(), (movecapture) );
					}
				}
				else if ( *tbit != movecapture )
				{
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );
				}
			}
		
			traybuttons = tmptraybuttons;
			tmptraybuttons.clear();
			movecapture = NULL;
			CreateLayout();
		}
	}
	
	if ( (button == NULL || fireClicksAlways) && !trayLButtonUp.empty() )
	{
		LSExecute(hWnd, trayLButtonUp.c_str(), SW_SHOWNORMAL);
	}

	movecapture = NULL;

	return button;
}

TrayButton* Tray::onMButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onMButtonDown(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button && (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		changeState = button;
	}
	else if ( button && (0x8000 & GetKeyState(VK_SHIFT)) )
	{
		string info = ("Icon Infos For Correct Hiding:\n\nClassname:\n\"" + GetTaskClass(button->hOwnerWnd) + "\"\n\nCurrent Windowtext:\n\"" + GetTaskWindowText(button->hOwnerWnd) + "\"");
		MessageBox(NULL, info.c_str(), "*xTrayHide Info Support!", MB_OK | MB_SETFOREGROUND);
	}
	
	if ( (button == NULL || fireClicksAlways) && !trayMButtonDown.empty() )
	{
		LSExecute(hWnd, trayMButtonDown.c_str(), SW_SHOWNORMAL);
	}

	return button;
}

TrayButton* Tray::onMButtonUp(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( button && button == changeState && (0x8000 & GetKeyState(VK_CONTROL)) )
	{
		if ( button->userHidden )
		{
			button->userHidden = false;
			
			if ( !trayOnIconUnHide.empty() )
			{
				CString exchange = trayOnIconUnHide.c_str();
				exchange.Replace("%[classname]%", GetTaskClass(button->hOwnerWnd).c_str());
				exchange.Replace("%[windowtext]%", GetTaskWindowText(button->hOwnerWnd).c_str());
				LSExecute(hWnd, exchange, SW_SHOWNORMAL);
			}
		}
		else
		{
			button->userHidden = true;
			if ( !trayRemove.empty() )
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);

			if ( !trayOnIconHide.empty() )
			{
				CString exchange = trayOnIconHide.c_str();
				exchange.Replace("%[classname]%", GetTaskClass(button->hOwnerWnd).c_str());
				exchange.Replace("%[windowtext]%", GetTaskWindowText(button->hOwnerWnd).c_str());
				LSExecute(hWnd, exchange, SW_SHOWNORMAL);
			}
		}

		updateHint(hWnd, (char *) button->strTooltip.c_str(), false);

		if ( autoSize )
			autoSizeFunction();
		
		CreateLayout();
	}
	
	if ( (button == NULL || fireClicksAlways) && !trayMButtonUp.empty() )
	{
		LSExecute(hWnd, trayMButtonUp.c_str(), SW_SHOWNORMAL);
	}

	changeState = NULL;

	return button;
}

TrayButton* Tray::onRButtonDblClk(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	return button;
}

TrayButton* Tray::onRButtonDown(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( (button == NULL || fireClicksAlways) && !trayRButtonDown.empty() )
	{
		LSExecute(hWnd, trayRButtonDown.c_str(), SW_SHOWNORMAL);
	}

	return button;
}

TrayButton* Tray::onRButtonUp(int x, int y)
{
	TrayButton *button = ButtonAtPoint( x, y );

	if ( (button == NULL || fireClicksAlways) && !trayRButtonUp.empty() )
	{
		LSExecute(hWnd, trayRButtonUp.c_str(), SW_SHOWNORMAL);
	}
	
	return button;
}

void Tray::onWheelDown(int x, int y)
{
	if(!trayMWheelDown.empty())
		LSExecute(hWnd, trayMWheelDown.c_str(), SW_SHOWNORMAL);
}

void Tray::onWheelUp(int x, int y)
{
	if(!trayMWheelUp.empty())
		LSExecute(hWnd, trayMWheelUp.c_str(), SW_SHOWNORMAL);
}

void Tray::onMouseEnter()
{
	if(!trayEnter.empty())
		LSExecute(hWnd, trayEnter.c_str(), SW_SHOWNORMAL);

	if ( showInfoTips && InfoTiphWnd )
		HideInfoTip();

	if (focusOnEnter)
	{	
		skipButton = true;
		mouse_event( MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MIDDLEDOWN,
						NULL, NULL, NULL, NULL );
		mouse_event( MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MIDDLEUP,
						NULL, NULL, NULL, NULL );
	}
}

void Tray::onMouseLeave()
{
	if (!trayLeave.empty())
		LSExecute(hWnd, trayLeave.c_str(), SW_SHOWNORMAL);
}

TrayButton* Tray::onMouseMove(int x, int y)
{
	if (!mouseInside)
	{
		mouseInside = true;
		onMouseEnter();
		SetTimer(hWnd, TIMER_MOUSETRACK, 100, 0);
	}

	TrayButton *button = ButtonAtPoint(x, y);

	return button;
}

void Tray::repaint()
{	
	if ( backupBitmap )
	{
		backupBitmap = (HBITMAP) SelectObject(backupDC, oldbackupBMP);
		DeleteObject(backupBitmap);
	}
	if ( backupDC )
		DeleteObject(backupDC);
	if ( oldbackupBMP )
		DeleteObject(oldbackupBMP);

	backupBitmap = 0;
	backupDC = 0;
	oldbackupBMP = 0;

	//Attention!! Background MUST exist
	if ( layeredFlag && IsOS(OS_2KXP) && traytexture && trayicon )
	{
		UpdateWindowTransparent(alphaTransparency);
	}
	else
	{
		RedrawWindow(hWnd, NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);

		if ( transparencyMode )
			updateRegion();
	}
}

void Tray::onPaint(HDC hDC)
{	
	int x = actualx;
	int y = actualy;

	int width = actualwidth;
	int height = actualheight;

	RECT rectPosition;
	rectPosition.left = 0;
	rectPosition.top = 0;
	rectPosition.right = width;
	rectPosition.bottom = height;

	HDC backgroundDC = 0;
	HBITMAP backgroundBitmap = 0;
	HBITMAP oldBMP = 0;

	// Use Complete Backup for WindowsInternal Repaint (Covering Windows, ...)
	if ( backupBitmap != 0 && backupDC != 0 )
	{
		BitBlt(hDC, 0, 0, width, height, backupDC, 0, 0, SRCCOPY);
	}
	else
	//1----------------------------------------------------------------
	{
	//1----------------------------------------------------------------
	
		backgroundDC = CreateCompatibleDC(NULL);

		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=width;      // Set size you need
		alpha.bmiHeader.biHeight=height;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;

		backgroundBitmap = CreateDIBSection( backgroundDC,
														&alpha,
														DIB_RGB_COLORS,
														&pvBits,
														NULL,
														0 );

		oldBMP = (HBITMAP) SelectObject(backgroundDC, backgroundBitmap);

		//True Transparency
		if (transparencyMode)
		{
			::SetBkColor(backgroundDC, RGB(255, 0, 255));
			::ExtTextOut(backgroundDC, 0, 0, ETO_OPAQUE, &rectPosition, NULL, 0, NULL);
		}
		//PrePaint Background (Fake Transparency)
		else
		{
			//  =======================================================================
			//  Paint desktop on display DC and then into background buffer
			//  =======================================================================
			PaintDesktopEx(backgroundDC, 0, 0, width, height, x, y, false);
		}
				
		//Normal Background Painting
		traytexture->apply(backgroundDC, 0, 0, width, height, transparencyMode);

		if ( !ghosttrayList.empty() )
		{
			for(GhostTrayListIterator glit = ghosttrayList.begin(); glit != ghosttrayList.end(); glit++)
			{
				if ( *glit != NULL )
				{
					int gtx = xParseCoord((*glit)->orgx, 0, width);
					int gty = xParseCoord((*glit)->orgy, 0, height);
					int gtwidth = xParseDimen((*glit)->orgwidth, width, width);
					int gtheight = xParseDimen((*glit)->orgheight, height, height);

					(*glit)->traytexture->applyOverlay(backgroundDC, gtx, gty, gtwidth, gtheight, (*glit)->alphatransparency );
				}
				else
				{
					break;
				}
			}
		}

		bool securecapture = false;
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{
			if ( (*tbit) != dontPaintAddedTrayIcon )
			{
				if ( ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) ) && (*tbit) != capture )
					(*tbit)->onPaint( backgroundDC, leftBorder, topBorder, width-rightBorder, height-bottomBorder );
				else if ( (*tbit) == capture )
					securecapture = true;
			}
		}

		if ( securecapture && ( !capture->userHidden || (capture->userHidden && hiddenvisible) ) )
			capture->onPaint( backgroundDC, leftBorder, topBorder, width-rightBorder, height-bottomBorder );
		
		// blt the double buffer to the display
		BitBlt(hDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);

		//Create BackupBitmap, if it doesn't exist!
		//----------------------------------------------------------------------
		if ( !skipBackup && backupBitmap == 0 && backupDC == 0 )
		{
			if ( backupDC == 0 )
				backupDC = CreateCompatibleDC(NULL);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=width;      // Set size you need
			alpha.bmiHeader.biHeight=height;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			backupBitmap = CreateDIBSection( backupDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

			oldbackupBMP = (HBITMAP) SelectObject(backupDC, backupBitmap);

			BitBlt(backupDC, 0, 0, width, height, backgroundDC, 0, 0, SRCCOPY);
		}
		//----------------------------------------------------------------------

	//1----------------------------------------------------------------
	}
	//1----------------------------------------------------------------

	if ( backgroundBitmap != 0 && backgroundDC != 0 )
	{
		backgroundBitmap = (HBITMAP) SelectObject(backgroundDC, oldBMP);
		DeleteObject(backgroundBitmap);
		DeleteDC(backgroundDC);
	}
}

void Tray::onTimer(int timerID)
{
	switch(timerID)
	{	
		case TIMER_MOUSETRACK:
		{
			POINT pt;
			GetCursorPos(&pt);
			
			RECT rc;
			GetWindowRect(hWnd, &rc);

			if ( transparencyMode )
			{
				HRGN hrgn = CreateRectRgn(0,0,0,0);
				int regionType = GetWindowRgn(hWnd, hrgn);
				if (regionType != ERROR) 
				{ 
					/* hrgn contains window region */ 
					ScreenToClient(hWnd, &pt);
					if(!PtInRegion(hrgn, pt.x, pt.y))
					{
						KillTimer(hWnd, TIMER_MOUSETRACK);

						mouseInside = false;
						onMouseLeave();	
						if ( showToolTip && ToolTips )
							updateHint(hWnd, "", false);
					
						if (capture)
							capture->useHover = false;

						repaint();	

						capture = NULL;
					
						DeleteObject(hrgn); /* finished with region */

						break;
					}
				}
				DeleteObject(hrgn); /* finished with region */
			}

			GetCursorPos(&pt);

			if(!PtInRect(&rc, pt))
			{
				KillTimer(hWnd, TIMER_MOUSETRACK);

				mouseInside = false;
				onMouseLeave();	
				if ( showToolTip && ToolTips )
					updateHint(hWnd, "", false);
			
				if (capture)
					capture->useHover = false;

				repaint();	

				capture = NULL;
				break;
			}

			ScreenToClient(hWnd, &pt);
			TrayButton *button = ButtonAtPoint( pt.x, pt.y );

			if ( button && button != capture )
			{
				if ( showToolTip && ToolTips )
				{
					string tooltip = button->strTooltip;
					if ( button->userHidden )
						tooltip.append("\nCurrent State In xTray: Hidden");
					updateHint(hWnd, (char *) tooltip.c_str(), false);
				}
							
				if (capture)
					capture->useHover = false;

				button->useHover = true;
				capture = button;
				repaint();
			}
			else if ( button == NULL && capture != NULL )
			{
				if ( showToolTip && ToolTips )
				{
					updateHint(hWnd, "", false);
				}
			
				capture->useHover = false;
				capture = NULL;
				repaint();
			}
		}
		break;

		case TIMER_ALPHAFADEIN:
		{
			if (UpdateAlpha <= alphaTransparency-alphaFadeStep)
			{
				isFading = true;

				UpdateAlpha += alphaFadeStep;
				UpdateWindowTransparent(UpdateAlpha);
				ShowWindow(hWnd, SW_SHOWNOACTIVATE);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEIN);
				if (UpdateAlpha != alphaTransparency)
					UpdateWindowTransparent(alphaTransparency);

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}
		break;

		case TIMER_ALPHAFADEOUT:
		{
			if (UpdateAlpha >= alphaFadeStep)
			{
				isFading = true;

				UpdateAlpha -= alphaFadeStep;
				UpdateWindowTransparent(UpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEOUT);
				UpdateAlpha = 0;
				ShowWindow(hWnd, SW_HIDE);

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}
		break;

		case TIMER_CLEAN:
		{
			bool clean = true;
			bool wascleaned = false;
			while ( clean )
			{
				clean = IconCleanUp();

				if ( clean )
				{
					wascleaned = true;

					if ( !trayRemove.empty() )
					{
						visibleIcons--;
						PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons ));

						LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);	
					}
				}
			}

			if ( wascleaned )
			{
				if ( autoSize )
					autoSizeFunction();
			
				CreateLayout();
			}
		}
		break;

		case TIMER_INFOTIPTIMEOUT:
		{
			KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
			bool secure = false;
			for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
			{
				if ( (*tbit) == UpdateConfig.button )
					secure = true;
			}

			if ( secure )
			{
				if ( IsWindow(UpdateConfig.button->hOwnerWnd) )
					ShowInfoTip(UpdateConfig.button, UpdateConfig.x, UpdateConfig.y);
			}
		}
		break;

		case INFOTIMER_AUTOHIDE:
		{
			HideInfoTip();	
		}	
		break;

		case TIMER_SECURESTART:
		{
			KillTimer(hWnd, TIMER_SECURESTART);
			infoTipsEnabled = true;
			SetTimer(hWnd, TIMER_CLEAN, cleanUpInterval, 0);
		}
		break;

		case TIMER_TRAYADD:
		{
			KillTimer(hWnd, TIMER_TRAYADD);
			if ( !trayAdd.empty() )
			{
				while ( internalAddCount > 0 )
				{
					if ( AddedTrayIcon != NULL )
					{
						if ( IsOnTrayList(AddedTrayIcon->hOwnerWnd, HideList, false) )
						{
							AddedTrayIcon->userHidden = true;
						}
						else
						{
							visibleIcons++;
							PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons ));
							LSExecute(hWnd, trayAdd.c_str(), SW_SHOWNORMAL);
						}
					}
					
					internalAddCount--;
				}
			}

			internalAddCount = 0;
			AddedTrayIcon = NULL;

			if ( autoSize )
				autoSizeFunction();
				
			CreateLayout();
		}
		break;


		case TIMER_TRAYREMOVE:
		{
			KillTimer(hWnd, TIMER_TRAYREMOVE);
			if ( !trayRemove.empty() )
			{
				if ( RemovedTrayIcon )
				{
					visibleIcons--;
					PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( visibleIcons ));
				}
				LSExecute(hWnd, trayRemove.c_str(), SW_SHOWNORMAL);	
			}

			RemovedTrayIcon = false;
						
			if ( autoSize )
				autoSizeFunction();
			
			CreateLayout();
		}
		break;

		case TIMER_ALPHAFADEUPBANG:
		{
			if (UpdateAlpha < alphaFadeBangEndValue-alphaFadeBangStep)
			{
				isFading = true;

				UpdateAlpha += alphaFadeStep;
				UpdateWindowTransparent(UpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
				if (UpdateAlpha != alphaFadeBangEndValue)
					UpdateWindowTransparent(alphaFadeBangEndValue);

				alphaTransparency = alphaFadeBangEndValue;

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}
		
		break;

		case TIMER_ALPHAFADEDOWNBANG:
		{
			if (UpdateAlpha > alphaFadeBangEndValue)
			{
				isFading = true;

				UpdateAlpha -= alphaFadeStep;
				UpdateWindowTransparent(UpdateAlpha);
			}
			else
			{
				KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
				if (UpdateAlpha != alphaFadeBangEndValue)
					UpdateWindowTransparent(alphaFadeBangEndValue);

				alphaTransparency = alphaFadeBangEndValue;

				isFading = false;

				if(fadeBackupBitmap)
				{
					fadeBackupBitmap = (HBITMAP) SelectObject(fadeBackupDC, oldfadeBackupBMP);
					DeleteObject(fadeBackupBitmap);
				}
				if (fadeBackupDC)
					DeleteDC(fadeBackupDC);
				if ( oldfadeBackupBMP )
					DeleteObject(oldfadeBackupBMP);

				fadeBackupBitmap = 0;
				fadeBackupDC = 0;
				oldfadeBackupBMP = 0;
			}
		}

		break;
	}
}

void Tray::onWindowPosChanging(WINDOWPOS *windowPos)
{
	if ( (windowPos->flags & SWP_NOZORDER) == 0 )
	{
		if ( !afterClass.empty() )
		{
			HWND before = FindWindowEx2(afterClass, afterTitle);
			if ( before != NULL )
			{
				if ( windowPos->hwndInsertAfter != before )
					windowPos->hwndInsertAfter = before;
			}
			else if ( !alwaysOnTop )
			{
				windowPos->flags = windowPos->flags | SWP_NOZORDER;
			}
		}
		else if ( !alwaysOnTop )
		{
			windowPos->flags = windowPos->flags | SWP_NOZORDER;
		}
	}
}

void Tray::onMove(int x, int y)
{
	PrefixedLSSetVariable(name, "CurrentX", IntToString( x ));
	PrefixedLSSetVariable(name, "CurrentY", IntToString( y ));
	
	actualx = x;
	actualy = y;

	if ( !transparencyMode && traytexture->isTransparencyNeeded() )
		repaint();

	if( !trayMove.empty() )
		LSExecute(hWnd, trayMove.c_str(), SW_SHOWNORMAL);
}

void Tray::onSize(int width, int height)
{
	PrefixedLSSetVariable(name, "CurrentWidth", IntToString( width ));
	PrefixedLSSetVariable(name, "CurrentHeight", IntToString( height ));

	actualwidth = width;
	actualheight = height;

	repaint();

	//Set new Rect for ToolTip Tracker
	if ( showToolTip && ToolTips )
		updateHint(hWnd, "", true);
			
	if( !trayResize.empty() )
		LSExecute(hWnd, trayResize.c_str(), SW_SHOWNORMAL);
}

BOOL Tray::onWindowMessage( UINT nMessage, WPARAM wParam, LPARAM lParam, LRESULT &lResult )
{
	lResult = 0;

	switch ( nMessage )
	{
		case LM_SYSTRAY:
		{
			switch ( wParam )
			{
				case NIM_ADD:
				{
					if ( IconAdd( (NOTIFYICONDATA*)lParam ) )
					{		
						if ( !blocked )
						{
							internalAddCount++;
							SetTimer(hWnd, TIMER_TRAYADD, 1, 0);
						}
						else
						{
							dontPaintAddedTrayIcon = NULL;
							AddedTrayIcon = NULL;
						}
						lResult = TRUE;
					}

					return TRUE;
				}

				case NIM_MODIFY:
				{
					if ( IconModify( (NOTIFYICONDATA*)lParam ) )
					{
						repaint();
						lResult = TRUE;
					}
					
					return TRUE;
				}

				case NIM_DELETE:
				{
					int result = IconDelete( (NOTIFYICONDATA*)lParam );

					if ( result == 1 )
					{
						SetTimer(hWnd, TIMER_TRAYREMOVE, 1, 0);
						lResult = TRUE;
					}
					else if ( result == -1 )
					{
						lResult = TRUE;
					}

					return TRUE;
				}

				case NIM_SETFOCUS:
				{
					lResult = TRUE;
					return TRUE;
				}

				case NIM_SETVERSION:
				{
					lResult = FALSE;
					return TRUE;
				}
			
				default:
				{
					lResult = FALSE;
					return TRUE;
				}
			}
		}

		case WM_CLOSE:
		{
			lResult = 0;
			return true;
		}

		case WM_DESTROY:
		{
			if(box)
			{
				if (hWnd && ToolTips)
				{
					updateHint(hWnd,"", false, true);
					DestroyWindow(ToolTips);
					ToolTips = 0;
				}

				if (hWnd)
				{
					KillTimer(hWnd, TIMER_MOUSETRACK);
					KillTimer(hWnd, TIMER_ALPHAFADEIN);
					KillTimer(hWnd, TIMER_ALPHAFADEOUT);
					KillTimer(hWnd, TIMER_CLEAN);
					KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
					KillTimer(hWnd, TIMER_ALPHAFADEUPBANG);
					KillTimer(hWnd, TIMER_ALPHAFADEDOWNBANG);
				}

				box = NULL;
				hide();
				
				ParentDestroyNotificationReceived = true;
			}
			
			return false;
		}

		case WM_LBUTTONDBLCLK:
		{
			TrayButton *button = onLButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_LBUTTONDOWN:
		{
			TrayButton *button = onLButtonDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam), lParam );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_LBUTTONUP:
		{
			TrayButton *button = onLButtonUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONDBLCLK:
		{
			TrayButton *button = onMButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONDOWN:
		{
			if ( skipButton )
				return true;

			TrayButton *button = onMButtonDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONDBLCLK:
		{
			TrayButton *button = onRButtonDblClk( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MBUTTONUP:
		{
			if ( skipButton )
			{
				skipButton = false;
				return true;
			}

			TrayButton *button = onMButtonUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONDOWN:
		{
			TrayButton *button = onRButtonDown( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_RBUTTONUP:
		{
			TrayButton *button = onRButtonUp( (int) (short) LOWORD( lParam ), (int) (short) HIWORD( lParam ) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);
			return true;
		}

		case WM_MOUSEWHEEL:
		{
			if ( (short)(HIWORD(wParam)) < 0 )
			{
				onWheelDown( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
				return true;
			}
			else
			{
				onWheelUp( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
				return true;
			}
		}

		case WM_MOUSEMOVE:
		{
			if ( ToolTips )
			{
				MSG hintMessage = {hWnd, nMessage, wParam, lParam};
				SendMessage( ToolTips, (UINT) TTM_RELAYEVENT, 0, (LPARAM) (LPMSG) &hintMessage ); 
			}

			TrayButton *button = onMouseMove( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			if ( button )
				button->onButtonClick(nMessage, wParam, lParam);

			return true;
		}

		case WM_PAINT:
		{
			if (visible)
			{
				PAINTSTRUCT ps;

				HDC hDC;

				hDC = BeginPaint( hWnd, &ps );

				onPaint( hDC );
			
				EndPaint( hWnd, &ps );
			}

			return true;
		}

		case WM_WINDOWPOSCHANGING:
		{
			onWindowPosChanging( (WINDOWPOS *) lParam );
			return false;
		}

		case WM_MOVE:
		{
			onMove( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			return true;
		}

		case WM_SIZE:
		{
			onSize( (int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam) );
			return true;
		}

		case WM_TIMER:
		{
			onTimer( (int) wParam );
			return true;
		}

		case WM_ENDSESSION:
		case WM_QUERYENDSESSION:
		{
			lResult = SendMessage(GetLitestepWnd(), nMessage, wParam, lParam);
			return true;
		}
	}

	return false;
}

TrayButton *Tray::ButtonAtPoint( int x, int y )
{
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
	{
		if ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) )
		{
			if ( (*tbit)->HasPoint(x, y) )
				return (*tbit);
		}
	}

	return NULL;
}

LRESULT WINAPI Tray::windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	Tray *tray = (Tray *) GetWindowLong(hWnd, 0);

	if (message == WM_NCCREATE)
	{
		LPCREATESTRUCT pcs = (LPCREATESTRUCT) lParam;
		tray = (Tray *) pcs->lpCreateParams;

		tray->hWnd = hWnd;
		SetWindowLong(hWnd, 0, (LONG) tray);
	}

	if (tray)
	{
		LRESULT lResult;

		if (tray->onWindowMessage(message, wParam, lParam, lResult))
			return lResult;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

void Tray::CreateLayout(bool clearBackground, int width, int height)
{
	if ( inCriticalProcess )
		return;

	inCriticalProcess = true;

	if ( width == -1 || height == -1 )
	{
		width = actualwidth;
		height = actualheight;
	}

	visibleIcons = 0;
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
	{
		if ( !(*tbit)->xTrayHideChecked )
		{
			if ( IsOnTrayList((*tbit)->hOwnerWnd, HideList, false) )
				(*tbit)->userHidden = true;

			(*tbit)->xTrayHideChecked = true;
		}

		(*tbit)->fixedPositionInTray = PosOnTrayList((*tbit)->hOwnerWnd, SortList, 100);

		if ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) )
			visibleIcons++;
	}

	SortTrayIcons(traybuttons);

	if ( startWithPosition > visibleIcons )
		startWithPosition = max(0, visibleIcons-1);

	int buttonCount = visibleIcons;

	PrefixedLSSetVariable(name, "CurrentIconCount", IntToString( buttonCount ));
	
	if (buttonCount <= 0)
	{
		PrefixedLSSetVariable(name, "CurrentLines", IntToString( lines_rows ));

		inCriticalProcess = false;
	
		if ( clearBackground )
			repaint();

		backupExportedButtonCount = buttonCount;

		if ( hideIfEmpty && !autoHidden && visible)
		{
			autoHidden = true;
			hide();
		}

		return;
	}
	else if ( hideIfEmpty && autoHidden && !visible )
	{
		autoHidden = false;
		show();
	}

	if (layout == 0)
	{
		//Start Horizontal Version
		//-----------------------------------------------------------------------
		int fireLineCount = 0;

		if ( wrapCount != -1 && !blocked )
		{
			if ( buttonCount/wrapCount >= lines_rows+lines_rowsAdded && buttonCount%wrapCount == 1 && backupExportedButtonCount < buttonCount )
			{
				lines_rowsAdded++;
				if ( lines_rows+lines_rowsAdded <= maxlines_rows )
				{	
					if ( !trayLineAdd.empty() )
						fireLineCount = 1;
				}
				else if ( lines_rows+lines_rowsAdded > maxlines_rows)
				{
					lines_rowsAdded--;
					wrapCount++;
				}
			}
			else if ( lines_rowsAdded >= 1 && buttonCount/wrapCount >= lines_rows && buttonCount%wrapCount == 0 && backupExportedButtonCount > buttonCount)
			{
				lines_rowsAdded--;
				if ( !trayLineRemove.empty() )
					fireLineCount = -1;
			}
		}

		int realLines = min(lines_rows+lines_rowsAdded, maxlines_rows);

		PrefixedLSSetVariable(name, "CurrentLines", IntToString( realLines ));

		if ( fireLineCount == -1 )
		{
			LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);

			inCriticalProcess = false;

			backupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}
		else if ( fireLineCount == 1 )
		{
			LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);

			inCriticalProcess = false;

			backupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}

		int x = 0;
		int y = 0;

		int buttoncounter = 1;
		int hcounter = 1;
		int vcounter = 1;

		int scrollcounter = 0;

		TrayButtonList tmptraybuttons;
		tmptraybuttons.clear();

		// set placement for each button
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{
			if ( startWithPosition > scrollcounter )
			{
				scrollcounter++;

				if ( !loopingTrayIcons )
					(*tbit)->setButtonLayout( -1, -1, -1, -1 );
				else
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );

				continue;
			}
			else
			{
				scrollcounter++;
			}

			if (direction == 1) //Right
				x = leftBorder + (hcounter-1) * (maxIconSize + xSpacing);
			else if (direction == 2) // Left
				x = (width - rightBorder) - hcounter * maxIconSize - (hcounter-1) * xSpacing;
			
			if ( wrapDirection == 1) //Bottom
				y = topBorder + (vcounter-1) * (maxIconSize + ySpacing);
			else if ( wrapDirection == 2) //Top
				y = (height - bottomBorder) - vcounter * maxIconSize - (vcounter-1) * ySpacing;

			(*tbit)->setButtonLayout( x, y, maxIconSize, maxIconSize );

			(*tbit)->positionInTray = buttoncounter;

			if ( wrapCount != -1 && buttoncounter%wrapCount == 0 )
			{
				hcounter = 0;
				vcounter++;
			}
			hcounter++;
			buttoncounter++;
		}

		if ( loopingTrayIcons && !tmptraybuttons.empty() )
		{
			for (TrayButtonListIterator tbit = tmptraybuttons.begin(); tbit != tmptraybuttons.end(); ++tbit)
			{
				if (direction == 1) //Right
					x = leftBorder + (hcounter-1) * (maxIconSize + xSpacing);
				else if (direction == 2) // Left
					x = (width - rightBorder) - hcounter * maxIconSize - (hcounter-1) * xSpacing;
				
				if ( wrapDirection == 1) //Bottom
					y = topBorder + (vcounter-1) * (maxIconSize + ySpacing);
				else if ( wrapDirection == 2) //Top
					y = (height - bottomBorder) - vcounter * maxIconSize - (vcounter-1) * ySpacing;

				(*tbit)->setButtonLayout( x, y, maxIconSize, maxIconSize );

				(*tbit)->positionInTray = buttoncounter;

				if ( wrapCount != -1 && buttoncounter%wrapCount == 0 )
				{
					hcounter = 0;
					vcounter++;
				}
				hcounter++;
				buttoncounter++;
			}
		}
		//-----------------------------------------------------------------------
		//End Horizontal Version
	}
	else if (layout == 1)
	{
		//Start Vertical Version
		//-----------------------------------------------------------------------
		int fireLineCount = 0;

		if ( wrapCount != -1 && !blocked )
		{
			if ( buttonCount/wrapCount >= lines_rows+lines_rowsAdded && buttonCount%wrapCount == 1 && backupExportedButtonCount < buttonCount )
			{
				lines_rowsAdded++;
				if ( lines_rows+lines_rowsAdded <= maxlines_rows )
				{	
					if ( !trayLineAdd.empty() )
						fireLineCount = 1;
				}
				else if ( lines_rows+lines_rowsAdded > maxlines_rows)
				{
					lines_rowsAdded--;
					wrapCount++;
				}
			}
			else if ( lines_rowsAdded >= 1 && buttonCount/wrapCount >= lines_rows && buttonCount%wrapCount == 0 && backupExportedButtonCount > buttonCount)
			{
				lines_rowsAdded--;
				if ( !trayLineRemove.empty() )
					fireLineCount = -1;
			}
		}

		int realLines = min(lines_rows+lines_rowsAdded, maxlines_rows);

		PrefixedLSSetVariable(name, "CurrentLines", IntToString( realLines ));

		if ( fireLineCount == -1 )
		{
			LSExecute(hWnd, trayLineRemove.c_str(), SW_SHOWNORMAL);
			
			inCriticalProcess = false;

			backupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}
		else if ( fireLineCount == 1 )
		{
			LSExecute(hWnd, trayLineAdd.c_str(), SW_SHOWNORMAL);
			
			inCriticalProcess = false;

			backupExportedButtonCount = buttonCount;

			CreateLayout();

			return;
		}

		int x = 0;
		int y = 0;

		int buttoncounter = 1;
		int hcounter = 1;
		int vcounter = 1;

		int scrollcounter = 0;

		TrayButtonList tmptraybuttons;
		tmptraybuttons.clear();

		// set placement for each button
		for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
		{	
			if ( startWithPosition > scrollcounter )
			{
				scrollcounter++;

				if ( !loopingTrayIcons )
					(*tbit)->setButtonLayout( -1, -1, -1, -1 );
				else
					tmptraybuttons.insert( tmptraybuttons.end(), (*tbit) );

				continue;
			}
			else
			{
				scrollcounter++;
			}

			if ( direction == 3 ) //bottom
				y = topBorder + (vcounter-1) * (maxIconSize + ySpacing);
			else if ( direction == 4)  //top
				y = (height - bottomBorder) - vcounter * maxIconSize - (vcounter-1) * ySpacing;
	
			if ( wrapDirection == 3 ) //right
				x = leftBorder + (hcounter-1) * (maxIconSize + xSpacing);
			else if ( wrapDirection == 4 ) //left
				x = (width - rightBorder) - hcounter * maxIconSize - (hcounter-1) * xSpacing;

			(*tbit)->setButtonLayout( x, y, maxIconSize, maxIconSize );

			(*tbit)->positionInTray = buttoncounter;

			if ( wrapCount != -1 && buttoncounter%wrapCount == 0 )
			{
				hcounter++;
				vcounter = 0;
			}
			vcounter++;
			buttoncounter++;
		}

		if ( loopingTrayIcons && !tmptraybuttons.empty() )
		{
			for (TrayButtonListIterator tbit = tmptraybuttons.begin(); tbit != tmptraybuttons.end(); ++tbit)
			{	
				if ( direction == 3 ) //bottom
					y = topBorder + (vcounter-1) * (maxIconSize + ySpacing);
				else if ( direction == 4)  //top
					y = (height - bottomBorder) - vcounter * maxIconSize - (vcounter-1) * ySpacing;
		
				if ( wrapDirection == 3 ) //right
					x = leftBorder + (hcounter-1) * (maxIconSize + xSpacing);
				else if ( wrapDirection == 4 ) //left
					x = (width - rightBorder) - hcounter * maxIconSize - (hcounter-1) * xSpacing;

				(*tbit)->setButtonLayout( x, y, maxIconSize, maxIconSize );

				(*tbit)->positionInTray = buttoncounter;

				if ( wrapCount != -1 && buttoncounter%wrapCount == 0 )
				{
					hcounter++;
					vcounter = 0;
				}
				vcounter++;
				buttoncounter++;
			}
		}
		//-----------------------------------------------------------------------
		//End Vertical Version
	}

	inCriticalProcess = false;

	if ( clearBackground )
		repaint();

	backupExportedButtonCount = buttonCount;
}

void Tray::ScrollIcons(bool next, int amount)
{
	if ( !autoSize )
	{
		if ( !loopingTrayIcons )
		{
			if (next)
			{
				if ( startWithPosition+amount < visibleIcons )
					startWithPosition = startWithPosition+amount;
			}
			else
			{
				if ( startWithPosition-amount < 0 )
					startWithPosition = 0;
				else
					startWithPosition = startWithPosition-amount;
			}	
		}
		else
		{
			if (next)
			{
				startWithPosition = startWithPosition+amount;
				if ( startWithPosition >= visibleIcons )
					startWithPosition = 0;
			}
			else
			{
				startWithPosition = startWithPosition-amount;
				if ( startWithPosition < 0 )
					startWithPosition = visibleIcons-1;
			}
		}

		CreateLayout();
	}
}

void Tray::autoSizeFunction()
{	
	int width = actualwidth;
	int height = actualheight;

	visibleIcons = 0;
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit)
	{
		if ( !(*tbit)->xTrayHideChecked )
		{
			if ( IsOnTrayList((*tbit)->hOwnerWnd, HideList, false) )
				(*tbit)->userHidden = true;

			(*tbit)->xTrayHideChecked = true;
		}
	
		if ( !(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible) )
			visibleIcons++;
	}

	int hcounter = 0;
	int vcounter = 0;

	int autoX = actualx;
	int autoY = actualy;
	int autoWidth = actualwidth;
	int autoHeight = actualheight;

	if (layout == 0)
	{
		if ( wrapCount != -1 )
		{
			hcounter = visibleIcons%wrapCount;	
			vcounter = min(visibleIcons/wrapCount, maxlines_rows);
			if ( vcounter > 1 && hcounter == 1 )
				CreateLayout(false);
		}
		else
		{
			hcounter = visibleIcons;
		}
		
		autoX = actualx;
		autoY = actualy;
	
		autoWidth = width;
		autoHeight = height;

		if (wrapCount == -1 || ( wrapCount != -1 && visibleIcons <= wrapCount ) )
		{
			autoWidth = leftBorder+(visibleIcons-1)*xSpacing+visibleIcons*maxIconSize+rightBorder;
			autoHeight = topBorder+maxIconSize+bottomBorder;
		}
		else if ( vcounter >= 1 )
		{
			autoWidth = leftBorder+(wrapCount-1)*xSpacing+wrapCount*maxIconSize+rightBorder;
			
			if ( visibleIcons%wrapCount == 0 || vcounter >= maxlines_rows )
			{
				autoHeight = topBorder+(vcounter-1)*ySpacing+vcounter*maxIconSize+bottomBorder;
			}
			else
			{
				autoHeight = topBorder+vcounter*ySpacing+(vcounter+1)*maxIconSize+bottomBorder;
			}
		}

		if ( direction == 2 )
		{
			autoX = actualx-(autoWidth-width);
		}
		if ( wrapDirection == 2 )
		{
			autoY = actualy-(autoHeight-height);
		}
			
		if ( actualx != autoX || actualy != autoY || actualwidth != autoWidth || actualheight != autoHeight )
		{
			reposition(3, autoX, autoY, autoWidth, autoHeight, autoSizeSteps, autoSizeDelay);
		}
		else
		{
			dontPaintAddedTrayIcon = NULL;
			AddedTrayIcon = NULL;
		}
	}
	else if (layout == 1)
	{
		if ( wrapCount != -1 )
		{
			vcounter = visibleIcons%wrapCount;
			hcounter = min(visibleIcons/wrapCount, maxlines_rows);
			if ( hcounter > 1 && vcounter == 1 )
				CreateLayout(false);
		}
		else
		{
			vcounter = visibleIcons;
		}

		autoX = actualx;
		autoY = actualy;
	
		autoWidth = width;
		autoHeight = height;

		if (wrapCount == -1 || ( wrapCount != -1 && visibleIcons <= wrapCount ) )
		{
			autoWidth = leftBorder+maxIconSize+rightBorder;
			autoHeight = topBorder+(visibleIcons-1)*ySpacing+visibleIcons*maxIconSize+bottomBorder;
		}
		else if ( hcounter >= 1 )
		{
			if ( visibleIcons%wrapCount == 0 || hcounter >= maxlines_rows)
			{
				autoWidth = leftBorder+(hcounter-1)*xSpacing+hcounter*maxIconSize+rightBorder;
			}
			else
			{
				autoWidth = leftBorder+hcounter*xSpacing+(hcounter+1)*maxIconSize+rightBorder;
			}
			autoHeight = topBorder+(wrapCount-1)*ySpacing+wrapCount*maxIconSize+bottomBorder;
		}

		if ( direction == 4 )
		{
			autoY = actualy-(autoHeight-height);
		}
		if ( wrapDirection == 4 )
		{
			autoX = actualx-(autoWidth-width);
		}

		if ( actualx != autoX || actualy != autoY || actualwidth != autoWidth || actualheight != autoHeight )
		{
			reposition(3, autoX, autoY, autoWidth, autoHeight, autoSizeSteps, autoSizeDelay);
		}
		/*else if ( actualwidth != autoWidth || actualheight != autoHeight )
		{
			reposition(2, 0, 0, autoWidth, autoHeight, autoSizeSteps, autoSizeDelay);
		}*/
		else
		{
			dontPaintAddedTrayIcon = NULL;
			AddedTrayIcon = NULL;
		}
	}
}

void Tray::updateHint(HWND hWnd, LPSTR caption, bool updatesize, bool remove)
{
	if ( !ToolTips )
		return;	
	
	if (!remove)
	{
		RECT clientRect;
		if ( updatesize )	
			GetClientRect(hWnd, &clientRect);

		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = TTF_TRANSPARENT;
		ti.hwnd = hWnd;
		ti.uId = 0;
		if ( updatesize )
			ti.rect = clientRect;
		ti.hinst = hInstance;
		ti.lpszText = caption;
		ti.lParam = 0;

		if (updatesize)
		{
			SendMessage(ToolTips, TTM_NEWTOOLRECT, 0, (LPARAM) (LPTOOLINFO) &ti);
		}
		else
		{
			SendMessage(ToolTips, TTM_UPDATETIPTEXT, 0, (LPARAM) (LPTOOLINFO) &ti);
		}
	}
	else
	{
		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uFlags = 0;
		ti.hwnd = hWnd;
		ti.uId = 0;
		SetRectEmpty(&ti.rect);
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		ti.lParam = 0;

		SendMessage(ToolTips, TTM_DELTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);
	}
}

bool Tray::IconAdd(NOTIFYICONDATA *pnid)
{
	if ( !IsWindow(pnid->hWnd) )
		return false;

	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit )
	{
		if ( (*tbit)->hOwnerWnd == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			return IconModify(pnid);
		}
	}

	TrayButton *button = new TrayButton( this, pnid->hWnd, pnid->uID );
	traybuttons.insert(traybuttons.end(), button);

	if ( (pnid->uFlags & NIF_MESSAGE) != 0 )
		button->uCallbackMessage = pnid->uCallbackMessage;

	if ( (pnid->uFlags & NIF_TIP) != 0 )
		button->strTooltip = (string) pnid->szTip;

	if ( (pnid->uFlags & NIF_ICON) != 0 )
		button->setIcon( pnid->hIcon );

	if ( (pnid->uFlags & NIF_INFO) != 0 && IsOS( OS_2KXP ) )
	{
		if ( (int) pnid->uTimeout >= 0 )
			button->iTimeOut = min( max((int) pnid->uTimeout, INFOTIP_MINIMUM_TIMEOUT), INFOTIP_MAXIMUM_TIMEOUT);
		
		button->strInfo = (string) pnid->szInfo;
		button->strTitle = (string) pnid->szInfoTitle;

		button->dwInfoFlags = pnid->dwInfoFlags;

		if ( infoTipsEnabled && showToolTip && showInfoTips )
		{
			UpdateConfig.x = button->buttonx+4;
			UpdateConfig.y = button->buttony+4;
			UpdateConfig.button = button;
			KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
			SetTimer(hWnd, TIMER_INFOTIPTIMEOUT, 1000, 0);
		}
	}

	if ( SortList.empty() && (autoSize || StrStrI(trayAdd.c_str(), "xtrayresize") != NULL || StrStrI(trayAdd.c_str(), "xtrayreposition") != NULL) )
		dontPaintAddedTrayIcon = button;

	AddedTrayIcon = button;

	return true;
}

bool Tray::IconModify(NOTIFYICONDATA *pnid)
{
	bool result = false;
	bool autoShowInfoTip = false;
	bool changeActiveTooltip = false;
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit )
	{	
		if ( (*tbit)->hOwnerWnd == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			if ( (*tbit)->Update(pnid, changeActiveTooltip, autoShowInfoTip) )
				result = true;

			if ( changeActiveTooltip )
				updateHint(hWnd, (char *) (*tbit)->strTooltip.c_str(), false);

			if ( (!(*tbit)->userHidden || ((*tbit)->userHidden && hiddenvisible)) && infoTipsEnabled && autoShowInfoTip && showToolTip && showInfoTips )
			{
				UpdateConfig.x = (*tbit)->buttonx+4;
				UpdateConfig.y = (*tbit)->buttony+4;
				UpdateConfig.button = (*tbit);
				KillTimer(hWnd, TIMER_INFOTIPTIMEOUT);
				SetTimer(hWnd, TIMER_INFOTIPTIMEOUT, 1000, 0);
			}

			return result;
		}
	}	

	return IconAdd(pnid);
}

int Tray::IconDelete(NOTIFYICONDATA *pnid)
{
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit )
	{	
		if ( (*tbit)->hOwnerWnd == pnid->hWnd && (*tbit)->uID == pnid->uID )
		{
			if ( UpdateConfig.button == (*tbit) )
				UpdateConfig.button = NULL;

			if ( capture == (*tbit) )
			{
				capture->useHover = false;
				capture = NULL;
			}

			int reallyHide = 1;

			if ( (*tbit)->userHidden && !hiddenvisible )
				reallyHide = -1;
			else
				RemovedTrayIcon = true;

			delete (*tbit);
			traybuttons.erase( tbit );
			
			return reallyHide;
		}
	}

	return 0;
}

bool Tray::IconCleanUp()
{
	for (TrayButtonListIterator tbit = traybuttons.begin(); tbit != traybuttons.end(); ++tbit )
	{	
		if ( !IsWindow((*tbit)->hOwnerWnd) )
		{
			if ( UpdateConfig.button == (*tbit) )
				UpdateConfig.button = NULL;

			if ( capture == (*tbit) )
			{
				capture->useHover = false;
				capture = NULL;
			}

			delete (*tbit);
			traybuttons.erase( tbit );
			
			return true;
		}
	}	
	
	return false;
}

bool Tray::CreateInfoTipWindow()
{
	bool br = false;

	InfoTiphWnd = CreateWindowEx( WS_EX_TOOLWINDOW | WS_EX_TOPMOST,
									TOOLTIPS_CLASS,
									TEXT(""),
									WS_POPUP | TTS_BALLOON | TTS_NOPREFIX | TTS_ALWAYSTIP,
									0, 0, 0, 0,
									0,
									0,
									hInstance,
									0);

	if ( InfoTiphWnd )
	{
		SetWindowLong(InfoTiphWnd, GWL_USERDATA, MAGIC_DWORD);

		SendMessage(InfoTiphWnd, TTM_SETMAXTIPWIDTH, 0, INFOTIP_MAXIMUM_WIDTH);

		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)InfoTiphWnd;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;
		ti.hinst = hInstance;
		ti.lpszText = NULL;
		SetRectEmpty(&ti.rect);

		// this fixes the mouse event bug on windows 2000
		SendMessage(InfoTiphWnd, CCM_SETVERSION, COMCTL32_VERSION, 0);

		SendMessage(InfoTiphWnd, (UINT) TTM_SETTIPBKCOLOR, (WPARAM) (COLORREF) InfoTipColor, 0 );   
		SendMessage(InfoTiphWnd, (UINT) TTM_SETTIPTEXTCOLOR, (WPARAM) (COLORREF) InfoTipTextColor, 0);  

		// add the toolinfo to the window
		br = SendMessage(InfoTiphWnd, TTM_ADDTOOL, 0, (LPARAM)(LPTOOLINFO)&ti);
	}
	
    return br;
}

void Tray::DestroyInfoTip()
{
	if (InfoTiphWnd)
	{
	    TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)InfoTiphWnd;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;
		
		SendMessage(InfoTiphWnd, TTM_DELTOOL, 0, (LPARAM)(LPTOOLINFO)&ti);

		KillTimer(hWnd, INFOTIMER_AUTOHIDE);
	
		DestroyWindow(InfoTiphWnd);
		InfoTiphWnd = NULL;
	}
}

void Tray::ShowInfoTip(TrayButton *button, int x, int y)
{
	if ( InfoTiphWnd )
	{
		RECT r;
		GetWindowRect(hWnd, &r);
		
		x = r.left+x;
		y = r.top+y;

		int icon = 0;
		// work out the icon
		if ( (button->dwInfoFlags & NIIF_INFO) != 0 )
			icon = 1;
		else if ( (button->dwInfoFlags & NIIF_WARNING) != 0 )
			icon = 2;
		else if ( (button->dwInfoFlags & NIIF_ERROR) != 0 )
			icon = 3;
		else if ( IsOS( OS_XP ) && (button->dwInfoFlags & NIIF_USER) != 0 )
			icon = (int) button->hIcon;
		else
			icon = 0;

		if (button->strTitle.empty())
			icon = 0;
	
		TOOLINFO ti;
		ti.cbSize = sizeof(TOOLINFO);
		ti.uId = (UINT)InfoTiphWnd;
		ti.hwnd = hWnd;
		ti.uFlags = TTF_TRACK | TTF_IDISHWND;

		string tooltip = "";
		if ( button->strInfo.empty() && button->strTitle.empty() )
			tooltip = button->strTooltip;
		else
			tooltip = button->strInfo;

		if ( button->userHidden )
			tooltip.append("\nCurrent State In xTray: Hidden");
			
		ti.lpszText = (char *) tooltip.c_str();

		ti.rect.left = r.left+button->buttonx;
		ti.rect.right = ti.rect.left+button->buttonwidth;
		ti.rect.top = r.top+button->buttony;
		ti.rect.bottom = ti.rect.top+button->buttonheight;

		SendMessage(InfoTiphWnd, TTM_SETTOOLINFO, 0, (LPARAM)(LPTOOLINFO)&ti);
			
		if ( button->strTitle.empty() )
			SendMessage(InfoTiphWnd, TTM_SETTITLE, (WPARAM) 0, (LPARAM) "\0");
		else
			SendMessage(InfoTiphWnd, TTM_SETTITLE, (WPARAM) icon, (LPARAM) button->strTitle.c_str());

		SendMessage(InfoTiphWnd, TTM_TRACKPOSITION, 0, (LPARAM) (DWORD) MAKELONG(x, y));
		SendMessage(InfoTiphWnd, TTM_TRACKACTIVATE, TRUE, (LPARAM) (LPTOOLINFO) &ti);

		// set the timer
		KillTimer(hWnd, INFOTIMER_AUTOHIDE);
		SetTimer(hWnd, INFOTIMER_AUTOHIDE, button->iTimeOut, NULL);
	}
}

void Tray::HideInfoTip()
{
	TOOLINFO ti;
	ti.cbSize = sizeof(TOOLINFO);
	ti.uId = (UINT)InfoTiphWnd;
	ti.hwnd = hWnd;
	ti.uFlags = TTF_TRACK | TTF_IDISHWND;

	SendMessage(InfoTiphWnd, TTM_TRACKACTIVATE, FALSE, (LPARAM) (LPTOOLINFO) &ti);

	KillTimer(hWnd, INFOTIMER_AUTOHIDE);
}
